@@ -65,8 +65,12 @@ extern tree_details_t *tree_details;
 #ifdef __GNUC__
 #define assert_not_reached()		G_STMT_START{						\
    {												\
-	FILE *f=fopen(g_build_filename(LOG_FILE,NULL),"a");			\
-	chdir(g_build_filename(LOG_DIR,NULL));			\
+	char *logdir=g_build_filename(LOG_DIR,NULL);						\
+	char *logfile=g_build_filename(LOG_FILE,NULL);						\
+	FILE *f=fopen(logfile,"a");								\
+        fprintf(stderr, "xffm: logfile = %s\n",logfile);					\
+        fprintf(stderr, "xffm: dumping core at= %s\n",logdir);					\
+	chdir(logdir);	g_free(logdir); g_free(logfile);					\
 	fprintf(f,"%s%s Core dump --> file %s: line %d (%s): should not be reached\n",		\
 			tod(),(g_get_prgname())?g_get_prgname():"?",				\
 			__FILE__,								\
@@ -76,16 +80,21 @@ extern tree_details_t *tree_details;
 	abort();										\
    };					}G_STMT_END
 #else /* !__GNUC__ */
+
 #define assert_not_reached()		G_STMT_START{						\
    {												\
-	FILE *f=fopen(g_build_filename(LOG_FILE,NULL),"a");			\
-	chdir(g_build_filename(LOG_DIR,NULL));					\
+	char *logdir=g_build_filename(LOG_DIR,NULL);						\
+	char *logfile=g_build_filename(LOG_FILE,NULL);						\
+	FILE *f=fopen(logfile,"a");								\
+        fprintf(stderr, "xffm: logfile = %s\n",logfile);					\
+        fprintf(stderr, "xffm: dumping core at= %s\n",logdir);					\
+	chdir(logdir);	g_free(logdir); g_free(logfile);					\
 	fprintf(f,"%s%s Core dump --> file %s: line %d (not GNUC): should not be reached\n",	\
 			tod(),(g_get_prgname())?g_get_prgname():"?",				\
 			__FILE__,								\
 			__LINE__);								\
 	fclose(f);										\
-	abort();										\
+	abort();		    								\
    };					}G_STMT_END
 #endif /* __GNUC__ */
 
@@ -530,7 +539,7 @@ extern tree_details_t *tree_details;
 
 
 #define ENABLE_RESIZE_FONTS	0x10000
-/*#define AUTOSIZE_NAME		0x20000*/
+/*#define SVG_ICONS		0x20000*/
 #define AUTOSCROLL		0x40000
 #define VERBOSE			0x80000 
 
