@@ -1,6 +1,6 @@
 
 /*
- * Copyright (C) 2002 Edscott Wilson Garcia
+ * Copyright (C) 2002-4 Edscott Wilson Garcia
  * EMail: edscott@imp.mx
  *
  * Copyright (C) 1998 Rasca, Berlin
@@ -42,34 +42,37 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
 
-#include "glade_callbacks.h"
-#include "glade_gui.h"
 #include "glade_support.h"
 
 #include "constants.h"
 #include "types.h"
+#include "modules.h"
+
+#include "dnd.h"
 
-#include "aux.h"
 #include "bookmarks.h"
 #include "cpy.h"
-#include "dnd.h"
+#include "rcp.h"
 #include "icons.h"
 #include "entry.h"
 #include "misc.h"
 #include "monitor.h"
-#include "smb_download.h"
-#include "smb_misc.h"
+#include "password_dialog.h"
 #include "run.h"
 #include "refresh.h"
 #include "uri.h"
 #include "widgets.h"
 
+/*#define DEBUG*/
+
 
 extern char *src_host;
 extern gboolean tar_extraction;
 static gboolean dragging = FALSE;
 static GtkTreeRowReference *target_ref = NULL;
 
+static int scrolltimer=0;
+
 enum
 {
     DND_NONE,
@@ -113,13 +116,82 @@ typedef struct selection_list_t
     GtkTreeRowReference *reference;
 }
 selection_list_t;
+typedef struct scrolltree_t
+{
+    GtkTreeView *treeview;
+    gint scrolly;
+    gint scrollheight;
+}
+scrolltree_t;
 
-void cancel_drop(gboolean state)
+static scrolltree_t scrolltree_v;
+static gint title_offset=0;
+
+G_MODULE_EXPORT
+void update_status_line (tree_entry_t *en){
+    if (en && en->path) {
+        gchar *readable_path=g_path_get_basename(en->path);
+        if (IS_NETWORK_TYPE(en->type)&&!IS_SAMBA_SERVER(en->subtype)){
+		ascii_readable(readable_path);
+	}
+	print_status_tmp(resolve_icon_small(en),readable_path,NULL);
+    	g_free(readable_path);
+	/*printf("DBG: item is=%s\n", en->path);*/
+    } 	
+    else print_status(NULL,"",NULL);
+}
+
+static void update_multistatus_line(tree_entry_t *en){
+    if (selection_list && g_list_length(selection_list) >= 1){
+	if (g_list_length(selection_list) == 1 && en) update_status_line(en);
+	else {
+	    gchar *g=g_strdup_printf(_("%d item(s) selected"),g_list_length(selection_list));
+	    if (g_list_length(selection_list) == 1) print_status ("xfce/info",g,NULL);
+	    else print_status ("xfce/warning",g,NULL);
+	    g_free(g);
+	}
+    } else if (en) update_status_line(en);
+}
+
+#define SCROLL_OFFSET (title_offset)
+static gint scrolltree(gpointer data){
+     GdkRectangle visible;
+     if (!GTK_IS_TREE_VIEW (scrolltree_v.treeview)) return TRUE;
+     
+     gtk_tree_view_get_visible_rect(scrolltree_v.treeview,&visible); 
+            /*printf("CELL: y=(%d,%d), visible=(%d,%d)\n",
+			    scrolltree_v.scrolly - scrolltree_v.scrollheight,
+			    scrolltree_v.scrolly + scrolltree_v.scrollheight,
+			    visible.y,visible.y+visible.height);*/
+     if (scrolltree_v.scrolly - scrolltree_v.scrollheight <= visible.y+SCROLL_OFFSET){   
+            /*printf("scroll UP \n");*/
+	    gtk_tree_view_scroll_to_point (scrolltree_v.treeview,0,scrolltree_v.scrolly - scrolltree_v.scrollheight-SCROLL_OFFSET);
+	    if (scrolltree_v.scrolly >= scrolltree_v.scrollheight) 
+		    scrolltree_v.scrolly -= scrolltree_v.scrollheight;
+     }
+     if (scrolltree_v.scrolly + scrolltree_v.scrollheight >= visible.y+visible.height){ 
+	    int y=  visible.y;
+	    /*printf("scroll DOWN \n");*/
+	    /*gtk_tree_view_scroll_to_point (scrolltree_v.treeview,0,scrolltree_v.scrolly + scrolltree_v.scrollheight);*/
+	    gtk_tree_view_scroll_to_point (scrolltree_v.treeview,0,visible.y+scrolltree_v.scrollheight);
+            gtk_tree_view_get_visible_rect(scrolltree_v.treeview,&visible); 
+	    if (y != visible.y){
+		    scrolltree_v.scrolly = visible.y + visible.height - scrolltree_v.scrollheight;
+	    }
+     }
+     return TRUE;
+}
+
+
+
+G_MODULE_EXPORT
+void cancel_drop (gboolean state)
 {
     drop_cancelled = state;
 }
 
-void reselect_dnd_list(GtkTreeView *treeview){
+G_MODULE_EXPORT
+void reselect_dnd_list (GtkTreeView *treeview){
     GList *tmp;
     GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
     for (tmp=selection_list;tmp;tmp=tmp->next){
@@ -131,9 +203,16 @@ void reselect_dnd_list(GtkTreeView *treeview){
 	   gtk_tree_path_free(treepath);
 	}
     }
+    /* for status line to be updated before release */
+    if (!(tree_details->preferences & ENABLE_GTK_CLICK)){
+       /* this is always true here...*/
+	if (selection_list && g_list_length(selection_list))
+    	    update_multistatus_line(NULL);
+    } 
 }
 
-void clear_dnd_selection_list(void)
+G_MODULE_EXPORT
+void clear_dnd_selection_list (void)
 {
     GList *tmp = selection_list;
     if (dragging) return;
@@ -157,7 +236,8 @@ void clear_dnd_selection_list(void)
     selection_list = NULL;
 }
 
-void clear_path_from_selection_list(GtkTreeView *treeview,GtkTreePath *treepath)
+G_MODULE_EXPORT
+void clear_path_from_selection_list (GtkTreeView *treeview,GtkTreePath *treepath)
 {
     GtkTreePath *path;
     GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
@@ -169,10 +249,10 @@ void clear_path_from_selection_list(GtkTreeView *treeview,GtkTreePath *treepath)
 	selection_list_t *sl;
 	sl = (selection_list_t *) tmp->data;
 	path=gtk_tree_row_reference_get_path(sl->reference);
-	if (gtk_tree_path_compare(path,treepath)==0){
+	if (!path || gtk_tree_path_compare(path,treepath)==0){
 	    selection_list = g_list_remove(selection_list,sl);
 	    g_free(sl);
-	    gtk_tree_path_free(path);
+	    if (path) gtk_tree_path_free(path);
 	    return;
 	}
         gtk_tree_path_free(path);
@@ -180,25 +260,50 @@ void clear_path_from_selection_list(GtkTreeView *treeview,GtkTreePath *treepath)
     return;
 }
 
-void   on_drag_begin(GtkWidget *widget,GdkDragContext *drag_context,gpointer user_data){
-    	dragging = TRUE;	
+G_MODULE_EXPORT
+void   on_drag_begin (GtkWidget *widget,GdkDragContext *drag_context,gpointer user_data){
+    	dragging = TRUE;
+	/*printf("DBG: 	on_drag_begin\n");*/
+	if (selection_list && g_list_length(selection_list) > 1){
+	    gtk_drag_set_icon_stock (drag_context,"gtk-dnd-multiple",-10,-10);
+	} else if (selection_list && g_list_length(selection_list) == 1){
+	    GdkPixbuf *pixbuf;
+	    selection_list_t *sl=(selection_list_t *)selection_list->data;
+	    pixbuf=resolve_icon_size(sl->en,BIG);
+	    if (pixbuf) {
+	      gtk_drag_set_icon_pixbuf (drag_context,pixbuf,-10,-10);
+	      g_object_unref(G_OBJECT(pixbuf));
+	    }
+	} else {
+	    gtk_drag_set_icon_stock (drag_context,"gtk-dnd",-10,-10);
+	}
+
+
 	/* this works better with plain motion-event */
 	/*reselect_dnd_list((GtkTreeView *)widget);   */
 }
 
 static gboolean valid_drop_site(GtkTreeView * treeview, gint x, gint y, tree_entry_t ** en, GtkTreeRowReference ** reference)
 {
-    tree_details_t *tree_details = get_tree_details(treeview);
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
     GtkTreeIter iter;
-    GtkTreePath *treepath;
-    int title_offset=0;
+    GtkTreePath *treepath=NULL;
 
 
+#ifdef DEBUG	
+	/*{
+	    gint nx,ny;
+	   gtk_tree_view_widget_to_tree_coords(treeview,x,y,&nx,&ny);
+	   printf("widget x,y=%d,%d  tree x,y=%d,%d\n",x,y,nx,ny);
+	}*/
+#endif
     if (gtk_tree_view_get_headers_visible(treeview)){
 	/* FIXME:    offset to title button assumed at fontheight+8 but should be
 	 *           widget vertical ascent, queried from the gtkcontainer child 
 	 *           (how do you do that?)
+	 *
+	 *           use gtk_tree_view_widget_to_tree_coords () ?
+	 *           no good.
 	 *           */
   	PangoRectangle logical_rect;
   	PangoLayout *layout = gtk_widget_create_pango_layout((GtkWidget *)treeview,"W");
@@ -206,23 +311,19 @@ static gboolean valid_drop_site(GtkTreeView * treeview, gint x, gint y, tree_ent
   	title_offset= PANGO_ASCENT(logical_rect) + PANGO_DESCENT(logical_rect);
   	g_object_unref(layout);
 	title_offset +=8;
-    } 
+    }
+    else title_offset=0;
     
-    /*{
-      GtkTreePath *tpath=gtk_tree_path_new_first        ();
-      GdkRectangle rect;
-      gtk_tree_view_get_cell_area     (treeview,
-                                             tpath,
-                                             NULL,
-                                             &rect);
-      gtk_tree_path_free(tpath);
-      printf("DBG: title offset at %d y=%d wy=%d height=%d\n",title_offset,y,rect.y,rect.height);
-    }*/
 
     if(!gtk_tree_view_get_path_at_pos(treeview, x, (tree_details->preferences & SHOW_TITLES) ? y - title_offset : y, &treepath, NULL, NULL, NULL))
     {
       nodrop:
-	    print_status(treeview, "xf_ERROR_ICON",
+	    
+	gtk_tree_view_set_drag_dest_row (treeview,NULL,GTK_TREE_VIEW_DROP_INTO_OR_BEFORE);
+
+	if (treepath) gtk_tree_path_free(treepath);
+#if 0
+	print_status("xfce/error",
 #ifdef ENOTSUP
 			    strerror(ENOTSUP),
 #else 
@@ -233,21 +334,31 @@ static gboolean valid_drop_site(GtkTreeView * treeview, gint x, gint y, tree_ent
 #endif
 #endif
 			    ": ",_("Drop"), NULL);
+#endif
 	return FALSE;
     }
+
+    /* scroll? */
+    {
+     GdkRectangle rect;
+     gtk_tree_view_widget_to_tree_coords (treeview,x,y,&(scrolltree_v.scrollheight),&(scrolltree_v.scrolly));
+     scrolltree_v.treeview=treeview;
+     gtk_tree_view_get_cell_area  (treeview, treepath, NULL,&rect);
+     scrolltree_v.scrollheight=rect.height;
+    }
+ 
+    
     if(*reference)
 	gtk_tree_row_reference_free(*reference);
     *reference = gtk_tree_row_reference_new(treemodel, treepath);
     gtk_tree_model_get_iter(treemodel, &iter, treepath);
-    gtk_tree_path_free(treepath);
+
     gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, en, -1);
 
     if(!en) goto nodrop;
 
     if(IS_BOOKMARK_TYPE((*en)->type) && IS_ROOT_TYPE((*en)->type))
 	    goto drop_ok;	
-    if(IS_APP_TYPE((*en)->type) && !IS_ROOT_TYPE((*en)->type))
-	    goto drop_ok;
     
     if(IS_DIR((*en)->type)||
        IS_NETDIR((*en)->subtype) ||
@@ -280,23 +391,31 @@ static gboolean valid_drop_site(GtkTreeView * treeview, gint x, gint y, tree_ent
     }
 
 drop_ok:
-    print_status(treeview, "xf_INFO_ICON", _("Drop"), ": ", abreviate((*en)->path), NULL);
+    gtk_tree_view_set_drag_dest_row (treeview,treepath,GTK_TREE_VIEW_DROP_INTO_OR_BEFORE);
+
+    if (treepath) gtk_tree_path_free(treepath);   
+#if 0 
+    print_status("xfce/info", _("Drop"), ": ", xffm_filename((*en)->path), NULL);
+#endif
     return TRUE;
 }
 
-char  *CreateSMBTmpList(GtkTreeView *treeview,
-		GList *in_list,char *target,gboolean samba_server){
+G_MODULE_EXPORT
+char  *CreateSMBTmpList (GList *in_list,char *target,gboolean samba_server){
     FILE *tmpfile;
     uri *u;
     char *w;
-    static char *fname=NULL;
+    gchar *fname=NULL;
     int nitems=0;
     struct stat s;
     GList *list;
     
     nitems=0;
     if ((fname=randomTmpName(NULL))==NULL) return NULL;
-    if ((tmpfile=fopen(fname,"w"))==NULL) return NULL;
+    if ((tmpfile=fopen(fname,"w"))==NULL) {
+	g_free(fname);
+	return NULL;
+    }
     fprintf(tmpfile,"cd /;cd \"%s\";\n",target);
     for (list=in_list;list;list=list->next){
         u = list->data;
@@ -305,6 +424,7 @@ char  *CreateSMBTmpList(GtkTreeView *treeview,
 		 if (!strchr(u->url,'/')) {
 			 fclose(tmpfile);
 			 unlink (fname);
+			 g_free(fname);
 			 return NULL;
 		 }
 		 w=g_strdup(strrchr(u->url,'/')+1);
@@ -315,10 +435,11 @@ char  *CreateSMBTmpList(GtkTreeView *treeview,
 		 }
 			 
 		 if (lstat(u->url,&s)<0){
-			 print_diagnostics(treeview,"xf_ERROR_ICON",
-					 strerror(errno),":",
+			 print_diagnostics("xfce/error",
+					 strerror(errno),":\n",
 					 u->url,"\n",NULL);
 			 fclose(tmpfile);
+			 g_free(fname);
 			 unlink (fname);
 			 g_free(w);
 			 w=NULL;
@@ -338,8 +459,8 @@ char  *CreateSMBTmpList(GtkTreeView *treeview,
     		   fprintf(tmpfile,"cd /;cd \"%s\";\n",target);
 		 }
 		 else { /*CHR, BLK, FIFO, LNK, SOCK */
- 			 print_diagnostics(treeview,"xf_ERROR_ICON",
-					 "cannot upload ",
+ 			 print_diagnostics("xfce/error",
+					  strerror(EBADF),":\n",
 					 u->url,"\n",NULL);
 	         }
 		 fflush(NULL);
@@ -351,6 +472,7 @@ char  *CreateSMBTmpList(GtkTreeView *treeview,
     /*fprintf(stderr,"dbg:nitems = %d\n",nitems);*/
     if (!nitems) {
        /*fprintf(stderr,"dbg:nitems = %d\n",nitems);*/
+	    g_free(fname);
 	    unlink(fname);
 	    return NULL;
     }
@@ -361,10 +483,10 @@ char  *CreateSMBTmpList(GtkTreeView *treeview,
 
 
 
-void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y, GtkSelectionData * data, guint info, guint time, void *client)
+G_MODULE_EXPORT
+void on_drag_data (GtkWidget * widget, GdkDragContext * context, gint x, gint y, GtkSelectionData * data, guint info, guint time, void *client)
 {
     GtkTreeView *treeview = (GtkTreeView *) widget;
-    tree_details_t *tree_details = get_tree_details(treeview);
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
     tree_entry_t *en;
     GtkTreeIter iter;
@@ -372,14 +494,16 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
     uri *u;
     int mode = 0;
     GList *tmp, *list = NULL;
-    char *tmpfile = NULL;
+    gchar *tmpfile = NULL;
     Atom atomo;
     gboolean will_expand=FALSE;
 
-    /*printf("DBG: on drag data\n"); */
+#ifdef DEBUG
+    printf("DBG: on drag data\n"); 
+#endif 
 
 
-    if(!set_load_wait(&tree_details))
+    if(!set_load_wait())
     {
 	gtk_drag_finish(context, FALSE, FALSE, time);
 	return;
@@ -387,7 +511,7 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
     if(!widget || data->length < 0 || data->format != 8)
     {
 	gtk_drag_finish(context, FALSE, FALSE, time);
-	goto drag_over;
+	return;
     }
 
     if(src_host){
@@ -404,12 +528,16 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
 	{
 	    src_host = g_strdup(property_data);
 	    XFree(property_data);
-	    /*printf("dbg: drop from host=%s\n",src_host); */
+#ifdef DEBUG
+	    printf("dbg: drop from host=%s\n",src_host); 
+#endif 
 	}
 	else
 	{
 	    src_host = g_strdup(our_host_name(treeview));
-	    /*printf("dbg: failed property get\n"); */
+#ifdef DEBUG
+	    printf("dbg: failed property get\n"); 
+#endif 
 	}
 
     }
@@ -424,10 +552,12 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
     if(!valid_drop_site(treeview, x, y, &en, &target_ref) || !en)
     {
 	gtk_drag_finish(context, FALSE, FALSE, time);
-	goto drag_over;
+	return;
+    }
+    if(!gtk_tree_row_reference_valid(target_ref)){
+	gtk_drag_finish(context, FALSE, FALSE, time);
+	return;
     }
-    if(!gtk_tree_row_reference_valid(target_ref))
-	goto drag_over;
     {
       GtkTreePath *treepath;
       treepath=gtk_tree_row_reference_get_path(target_ref);
@@ -435,15 +565,10 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
       gtk_tree_path_free(treepath);
     }
     
-    /* entry comes from valid_drop_site(), this gives us the iter*/
-    /*if(!get_entry_from_reference(treeview, target_ref, &iter, &en))
-	goto drag_over;*/
-	
-
     if(!(info == TARGET_STRING) && !(info == TARGET_URI_LIST))
     {
 	gtk_drag_finish(context, FALSE, FALSE, time);
-	goto drag_over;
+	return;
     }
 
 
@@ -455,29 +580,37 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
 	the_mode = mode = TR_LINK;
     else
     {
-	print_status(treeview, "xf_ERROR_ICON",strerror(EINVAL), NULL);
-	goto drag_over;
+	print_status("xfce/error",strerror(EINVAL), NULL);
+	gtk_drag_finish(context, FALSE, FALSE, time);
+	return;
     }
 
     if (IS_EXPANDED(en->type)) will_expand=TRUE;
 
 
-    cursor_wait(treeview);
-    /*printf("DBG:drag data=%s\n",(const char *) data->data); */
+    cursor_wait();
     nitems = uri_parse_list((const char *)data->data, &list);
-    /*printf ("dbg:nitems=%d\n",nitems); */
-    if(!nitems)
+#ifdef DEBUG
+    printf("DBG:drag data=%s\n",(const char *) data->data); 
+    printf ("dbg:nitems=%d\n",nitems); 
+#endif    
+    if(!nitems){
+	gtk_drag_finish(context, FALSE, FALSE, time);
 	goto drag_over;		/* of course */
+    }
     u = list->data;
     
     if(u->type == URI_SMB)
     {	/* src_host may be remote or local, but target only local
 	  or bookmark */		
       if (IS_DIR(en->type)) {    
-	SMBGetFile(treeview, en->path, list);	
+#ifdef USE_SMB_BRANCH
+	XF_SMBGetFile(treeview, en->path, list);
+#endif
       } 
       if (!(IS_BOOKMARK_TYPE(en->type) && IS_ROOT_TYPE(en->type))){
         list = uri_free_list(list);
+	gtk_drag_finish(context, FALSE, FALSE, time);
         goto drag_over;
       }
     }
@@ -490,9 +623,12 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
 	{
 	    u = tmp->data;
 	    add2bookmarks(treeview, u->url);
-	    /*printf("DBG: adding %s to bookmarks\n",u->url);*/
+#ifdef DEBUG
+	    printf("DBG: adding %s to bookmarks\n",u->url);
+#endif 
 	}
 	list = uri_free_list(list);
+	gtk_drag_finish(context, FALSE, FALSE, time);
 	goto drag_over;
     }
 
@@ -510,54 +646,35 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
 	target=g_strdup(strstr(p,"/"));
         if (IS_NETFILE(en->subtype)) *strrchr(target,'/')=0;
       } 
-      /*printf("DBG:target=%s\n",target);*/
+#ifdef DEBUG
+     printf("DBG:target=%s\n",target);
+#endif 
       
-      tmpfile=CreateSMBTmpList(treeview,list,target,
+      tmpfile=CreateSMBTmpList(list,target,
 		      IS_SAMBA_SERVER(en->subtype));
       g_free(target);
       target=NULL;
       if (!tmpfile) {
+#ifdef DEBUG
+         fprintf(stderr,"dbg:null tmpfile\n");
+#endif 
          /*fprintf(stderr,"dbg:null tmpfile\n");*/
          list=uri_free_list (list);
+	 gtk_drag_finish(context, FALSE, FALSE, time);
          goto drag_over;
       }
-      /*else fprintf(stderr,"dbg:tmpfile=%s\n",tmpfile);*/
-      SMBDropFile (treeview,en,&iter,tmpfile);
+#ifdef DEBUG
+      else fprintf(stderr,"dbg:tmpfile=%s\n",tmpfile);
+#endif 
+#ifdef USE_SMB_BRANCH
+      XF_SMBDropFile (treeview,en,&iter,tmpfile);
+#endif 
       list = uri_free_list(list);
+      gtk_drag_finish(context, TRUE, FALSE, time);
       goto drag_over;
     }
 
 
-    if(IS_APP_TYPE(en->type) && !IS_ROOT_TYPE(en->type))
-    {
-	for(tmp = list; tmp != NULL; tmp = tmp->next)
-	{  
-	    gchar *in_cmd;
-	    
-	    u = tmp->data;
-	    
-	    if (en->filter) {
-		in_cmd = g_strconcat(en->path," ",en->filter,NULL);
-	    	print_status(treeview, "xf_INFO_ICON", 
-			       _("Executing"), " : ", 
-			        FILENAME(en)," ",en->filter,
-			       NULL);
-	    }
-	    else {
-		in_cmd = g_strdup(en->path);
-	    	print_status(treeview, "xf_INFO_ICON", 
-			       _("Executing"), " : ", 
-			        FILENAME(en),
-			       NULL);
-	    }
-	    
- 	    on_run_path(treeview, in_cmd, u->url, IS_IN_TERM(en->subtype),
-			   FALSE, FALSE);
-	    g_free(in_cmd);
-	}
-	list = uri_free_list(list);
-	goto drag_over;
-    }
 
     if(strcmp(src_host, our_host_name(treeview)) != 0)
     {
@@ -566,41 +683,26 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
 
 	l=g_list_length(list);
 	srcs = (char **)malloc((l+1) * sizeof(char *));
-	if(!srcs) g_assert_not_reached();
+	if(!srcs) assert_not_reached();
 	srcs[l] = NULL;
 	for(l = 0, tmp = list; tmp != NULL; tmp = tmp->next)
 	{
 	    u = tmp->data;
 	    srcs[l] = u->url;
 	    srcs[l + 1] = NULL;
-	    /* with rsync, one by one */
-	    if(!(tree_details->preferences & RSYNC_X_SCP))
-	    {
-		rsync(treeview, srcs + l, en->path);
-	    }
 	    l++;
 	}
-	if(tree_details->preferences & RSYNC_X_SCP)
-	    rsync(treeview, srcs, en->path);
+	rsync(srcs, en->path);
 	g_free(srcs);
 	srcs=NULL;
 
-#if 0
-	for(tmp = list; tmp != NULL; tmp = tmp->next)
-	{
-	    u = tmp->data;
-	    if(!rsync(treeview, u->url, en->path))
-		break;
-	}
-#endif
 	list = uri_free_list(list);
+    	gtk_drag_finish(context, TRUE, FALSE, time);
 	goto drag_over;
     }
 
     /* above: u = list->data; */
-    /*fprintf(stderr,"dbg:dnd, src=%s(tarchild=%d) tgt=%s\n",
-     *  u->url,t_en->type & FT_TARCHILD,t_en->path);*/
-    /*fprintf(stderr,"dbg:dnd, src=%s tgt=%s\n",u->url,en->path); */
+ 
 
     {
 	/* nonsense check */
@@ -609,8 +711,9 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
 	if(st.st_ino == en->st->st_ino)
 	{
 	    list = uri_free_list(list);
-	    print_status(treeview, "xf_ERROR_ICON", _("Source and target are the same!"), NULL);
+	    print_status("xfce/error", strerror(EEXIST), NULL);
 	    /*fprintf(stderr,"dbg:nonsense 1\n"); */
+    	    gtk_drag_finish(context, FALSE, FALSE, time);
 	    goto drag_over;
 	}
 	if(!S_ISDIR(st.st_mode) && strchr(u->url, '/'))
@@ -623,8 +726,9 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
 		list = uri_free_list(list);
 		g_free(p);
 		p=NULL;
-		print_status(treeview, "xf_ERROR_ICON", _("Source and target are the same!"), NULL);
+		print_status("xfce/error", strerror(EEXIST), NULL);
 		/*fprintf(stderr,"dbg:nonsense 2\n"); */
+    	        gtk_drag_finish(context, FALSE, FALSE, time);
 		goto drag_over;
 	    }
 	    g_free(p);
@@ -635,53 +739,45 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
     /* tmpfile ==NULL means drop cancelled */
 
 
-    tmpfile = CreateTmpList(treeview, list, en);
-    /*fprintf(stderr,"dbg:dnd, tmpfile=%s\n",tmpfile); */
+    tmpfile = CreateTmpList(list, en);
+
     if(!tmpfile)
     {
 	char *what;
-	if(mode == TR_MOVE)
-	    what = _("moved");
-	else if(mode == TR_COPY)
-	    what = _("copied");
-	else
-	    what = _("linked");
-	/*printf("DBG:null tmpfile\n"); */
-	print_status(treeview, "xf_INFO_ICON", _("Nothing"), " ", what, NULL);
+	if(mode == TR_MOVE) what = _("Nothing moved");
+	else if(mode == TR_COPY) what = _("Nothing copied");
+	else what = _("Nothing linked");
+#ifdef DEBUG
+	printf("DBG:null tmpfile\n"); 
+#endif 
+	print_status("xfce/info", what, NULL);
 	list = uri_free_list(list);
+    	gtk_drag_finish(context, FALSE, FALSE, time);
 	goto drag_over;
     }
+#ifdef DEBUG
+     else fprintf(stderr,"dbg:dnd, tmpfile=%s\n",tmpfile); 
+#endif 
     /* acording to tmpfile name, do a direct move, here, and break.- */
 
+    IndirectTransfer(mode, tmpfile);
 
-    /*FIXME: uncomment when tar module is added 
-       if (tar_extraction) DirectTransfer(ctree,mode,tmpfile); 
-       else */
-    {
-	if((mode & TR_LINK) || (on_same_device() && (mode & TR_MOVE)))
-	{
-	    /*printf("dbg:going to Direct transfer\n");*/
-	    DirectTransfer(treeview, mode, tmpfile);
-	}
-	else
-	{
-	    /*printf("dbg:going to Indirect transfer\n");*/
-	    IndirectTransfer(treeview, mode, tmpfile);
-	}
-    }
-
-    if(tmpfile)
-	unlink(tmpfile);
-
+    if(tmpfile) unlink(tmpfile);
+    g_free(tmpfile);
+    
     list = uri_free_list(list);
 
 
-    /*fprintf(stderr,"dbg:parent:runOver\n"); */
+#ifdef DEBUG
+    fprintf(stderr,"dbg:parent:runOver\n"); 
+#endif 
+
+    gtk_drag_finish(context, TRUE, (the_mode & TR_MOVE) ? TRUE : FALSE, time);
 
   drag_over:
-    update_dir(treeview, target_ref);
+    /*update_dir(treeview, target_ref);*/
     /*tar_extraction=FALSE;*/
-    unset_load_wait(&tree_details);
+    unset_load_wait();
     if (will_expand && (IS_NETDIR(en->subtype) || 
 			IS_XF_NETSHARE(en->subtype)||  
 			IS_NETFILE(en->subtype)) )
@@ -691,19 +787,23 @@ void on_drag_data(GtkWidget * widget, GdkDragContext * context, gint x, gint y,
     	gtk_tree_path_free(treepath);
     }
 
-    hide_stop(tree_details->window);
-    cursor_reset(treeview);
-/* some files may or may not succeed in move, but we'll let monitor fix up later */
-    gtk_drag_finish(context, TRUE, (the_mode & TR_MOVE) ? TRUE : FALSE, time);
-    if (!IS_BOOKMARK_TYPE(en->type)) local_monitor(treeview,TRUE);
+
+    /*if (!IS_BOOKMARK_TYPE(en->type)) */
+    local_monitor(TRUE);
+    cursor_reset();
+#ifdef DEBUG
+    fprintf(stderr,"dbg:and done... \n"); 
+#endif 
 }
 
-gboolean is_in_dnd_selection(GtkTreePath * path){
+G_MODULE_EXPORT
+gboolean is_in_dnd_selection (GtkTreePath * path){
 	GList *tmp;
 	if (!path) return TRUE;
 	for (tmp=selection_list;tmp;tmp=tmp->next){
 		selection_list_t *sl=tmp->data;
            	GtkTreePath *treepath = gtk_tree_row_reference_get_path(sl->reference);
+		if (!treepath) continue;
 		if(gtk_tree_path_compare (path,treepath)==0){
 	           gtk_tree_path_free(treepath);
 		   return TRUE;
@@ -713,66 +813,80 @@ gboolean is_in_dnd_selection(GtkTreePath * path){
 	return FALSE;
 }
 
-void get_dnd_selection(GtkTreeModel * treemodel, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
+G_MODULE_EXPORT
+void get_dnd_selection (GtkTreeModel * treemodel, GtkTreePath * path, GtkTreeIter * iter, gpointer data)
 {
-    GtkTreeView *treeview = (GtkTreeView *) data;
-    GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
     tree_entry_t *en;
 
-   /* if (drag_type==DRAG_TYPE_INCONSISTENT) return;*/
+    /*if (drag_type==DRAG_TYPE_INCONSISTENT) return;*/
 
     gtk_tree_model_get(treemodel, iter, ENTRY_COLUMN, &en, -1);
-    if(!IS_PATH(en->type) && !IS_NETTHING(en->subtype) )
-    {
-	gtk_tree_selection_unselect_iter(selection, iter);
-	return;
-    }
+    if(!IS_PATH(en->type) && !IS_NETTHING(en->subtype) ) return;
+    
     if (!IS_LOCAL_TYPE(en->type) && IS_ROOT_TYPE(en->type)) return;
-    /*if((IS_PATH(en->type) && drag_type==DRAG_TYPE_NET) ||
-       (IS_NETTHING(en->subtype) && drag_type==DRAG_TYPE_LOCAL)){
-        drag_type=DRAG_TYPE_INCONSISTENT;
-	print_diagnostics(treeview,"xf_ERROR_ICON",strerror(E2BIG),"\n",NULL); 
-	return;
-    }*/
+    
     if(IS_PATH(en->type)) drag_type=DRAG_TYPE_LOCAL;
     if(IS_NETTHING(en->subtype)) drag_type=DRAG_TYPE_NET;
+    
 
-    {
+    if (en && en->path) {
 	selection_list_t *sl;
+	GList *tmp;
+	for (tmp=selection_list;tmp;tmp=tmp->next){
+	    sl = (selection_list_t *)tmp->data;
+	    if (strcmp(sl->en->path,en->path)==0) return;
+	}
 	sl = (selection_list_t *) malloc(sizeof(selection_list_t));
-	if(!sl)
-	    g_assert_not_reached();
+	if(!sl) assert_not_reached();
 	sl->en = en;
 	sl->reference = gtk_tree_row_reference_new(treemodel, path);
 	selection_list = g_list_append(selection_list, sl);
-/*	selection_list = g_list_prepend(selection_list, sl);*/
-	/*printf("DBG: added to selection list=%s\n",en->path);*/
-    }
+	
+	
+#ifdef DEBUG
+	printf("DBG:get_dnd_selection() added to selection list=%s\n",en->path);
+#endif 
+    } 
+    if (tree_details->preferences & ENABLE_GTK_CLICK) update_multistatus_line(en);
+    
+    
     return;
 }
 
+#ifdef DEBUG
+G_MODULE_EXPORT
+void the_sl (void)
+    {
+	selection_list_t *sl;
+	GList *tmp;
+	printf("****************************\n");
+	for (tmp=selection_list;tmp;tmp=tmp->next) {
+	    sl = (selection_list_t *)tmp->data;
+	    printf("-> %s\n",sl->en->path);
+	}
+    }
+#endif 
+
 
 /*
  * DND sender: prepare data for the remote
  * event: drag_data_get
  */
-void on_drag_data_get(GtkWidget * widget, GdkDragContext * context, GtkSelectionData * selection_data, guint info, guint time, gpointer data)
+G_MODULE_EXPORT
+void on_drag_data_get (GtkWidget * widget, GdkDragContext * context, GtkSelectionData * selection_data, guint info, guint time, gpointer data)
 {
     char *files;
     GList *tmp;
 
-    GtkTreeView *treeview = (GtkTreeView *) widget;
-    GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
-    
     if(!widget)	return;
-    /*if(selection_list)	clear_dnd_selection_list();*/
     
-    /*gtk_tree_selection_selected_foreach(selection, get_dnd_selection, (gpointer) treeview);*/
-    /*drag_type=DRAG_TYPE_UNDEFINED;*/
-    if(!selection_list) return;
-   
-    if(!(drag_type&(DRAG_TYPE_LOCAL|DRAG_TYPE_NET))) 
+    if(!selection_list) {
 	    return;
+    }
+   
+    if(!(drag_type&(DRAG_TYPE_LOCAL|DRAG_TYPE_NET))){
+	    return;
+    }
 
 
     /* prepare data for the receiver */
@@ -787,7 +901,7 @@ void on_drag_data_get(GtkWidget * widget, GdkDragContext * context, GtkSelection
 	 }
 	 switch (drag_type){
 	  default:
-		  g_assert_not_reached();
+		  assert_not_reached();
 	  case DRAG_TYPE_LOCAL:	    
 	    selection_len = 0;
 	    for (tmp = selection_list;tmp;tmp = tmp->next) {
@@ -829,7 +943,7 @@ void on_drag_data_get(GtkWidget * widget, GdkDragContext * context, GtkSelection
 	    }
 	    
     	    dnd_data = files = g_malloc (selection_len + 1);
-    	    if (!files) g_assert_not_reached();
+    	    if (!files) assert_not_reached();
     	    files[0]=0;
 
 	    for (tmp = selection_list;tmp;tmp = tmp->next) {
@@ -867,13 +981,15 @@ void on_drag_data_get(GtkWidget * widget, GdkDragContext * context, GtkSelection
     gtk_selection_data_set(selection_data, selection_data->target, 
 		    8, (const guchar *)dnd_data, selection_len); 
     dragging=FALSE;
-    /*printf("DBG drag get done\n");    */
+#ifdef DEBUG
+    printf("DBG drag get done\n");    
+#endif 
 }
 
-gboolean on_drag_motion(GtkWidget * widget, GdkDragContext * dc, gint x, gint y, guint t, gpointer data)
+G_MODULE_EXPORT
+gboolean on_drag_motion (GtkWidget * widget, GdkDragContext * dc, gint x, gint y, guint t, gpointer data)
 {
     GtkTreeView *treeview = (GtkTreeView *) widget;
-    tree_details_t *tree_details = (tree_details_t *) get_tree_details(treeview);
     tree_entry_t *en;
     GdkDragAction action;
 
@@ -888,12 +1004,14 @@ gboolean on_drag_motion(GtkWidget * widget, GdkDragContext * dc, gint x, gint y,
 	action = GDK_ACTION_MOVE;
 
 
-
+    if (!scrolltimer){ 
+		scrolltimer = g_timeout_add_full(0, 150, (GtkFunction) scrolltree, (gpointer) treeview, NULL);
+		/*printf("DBG: scrolltimer=%d\n",scrolltimer);*/
+    }
 
     if(!valid_drop_site(treeview, x, y, &en, &target_ref) || !en)
     {				/*change icon */
 	gdk_drag_status(dc, 0, t);
-
     }
     else if(dc->actions == GDK_ACTION_MOVE)
 	gdk_drag_status(dc, GDK_ACTION_MOVE, t);
@@ -922,16 +1040,18 @@ void on_drag_data_delete(GtkWidget * widget, GdkDragContext * context, gpointer
 }
 #endif
 
-void on_drag_end(GtkWidget * widget, GdkDragContext * context, gpointer data)
+G_MODULE_EXPORT
+void on_drag_end (GtkWidget * widget, GdkDragContext * context, gpointer data)
 {
-    GList *tmp;
-    GtkTreeView *treeview = (GtkTreeView *) widget;
-    tree_details_t *tree_details = get_tree_details(treeview);
-
-    /*printf("DBG drag end\n");*/
+    /*GList *tmp;
+    GtkTreeView *treeview = (GtkTreeView *) widget;*/
+#ifdef DEBUG
+    printf("DBG drag end\n");
+#endif
     dragging = FALSE;
-    if(!widget)
-	return;
+    /*if (scrolltimer)g_source_remove(scrolltimer);
+    scrolltimer=0,scrollx=-1, scrolly=-1;*/
+    if(!widget)	return;
 
     if(dnd_data)
     {
@@ -939,11 +1059,14 @@ void on_drag_end(GtkWidget * widget, GdkDragContext * context, gpointer data)
 	dnd_data = NULL;
     }
 
-    if(!set_load_wait(&tree_details))
+    if(!set_load_wait())
     {
 	printf("DBG: cannot set tree_details->loading! (on_drag_end())\n");
 	return;
     }
+    
+#if 0
+     this looks broken
     for (tmp=selection_list;tmp;tmp=tmp->next)
     {
 	selection_list_t *sl;
@@ -952,6 +1075,7 @@ void on_drag_end(GtkWidget * widget, GdkDragContext * context, gpointer data)
 		update_dir(treeview, sl->reference);
 	}
     }
+#endif
 
     /* this would unselect selection after drop: */
 #if 0
@@ -968,17 +1092,33 @@ void on_drag_end(GtkWidget * widget, GdkDragContext * context, gpointer data)
 #endif
 
     
-    unset_load_wait(&tree_details);
+    unset_load_wait();
+#ifdef DEBUG
+    printf("DBG drag end done\n");
+#endif
     return;
 }
 
-void setup_drag_signal(GtkTreeView * treeview)
+G_MODULE_EXPORT
+void  on_drag_leave (GtkWidget *widget, GdkDragContext *drag_context, guint time, gpointer user_data){
+#ifdef DEBUG
+	printf("on_drag_leave*************\n");
+#endif
+   
+   if (scrolltimer)g_source_remove(scrolltimer);
+   scrolltimer=0;
+   /*print_status(NULL," ",NULL);*/
+}
+
+G_MODULE_EXPORT
+void setup_drag_signal (GtkTreeView * treeview)
 {
     g_signal_connect(G_OBJECT(treeview), "drag_data_received", G_CALLBACK(on_drag_data), NULL);
     g_signal_connect(G_OBJECT(treeview), "drag_data_get", G_CALLBACK(on_drag_data_get), NULL);
     g_signal_connect(G_OBJECT(treeview), "drag_motion", G_CALLBACK(on_drag_motion), NULL);
     g_signal_connect(G_OBJECT(treeview), "drag_end", G_CALLBACK(on_drag_end), NULL);
     g_signal_connect(G_OBJECT(treeview), "drag_begin", G_CALLBACK(on_drag_begin), NULL);
+    g_signal_connect(G_OBJECT(treeview), "drag-leave", G_CALLBACK(on_drag_leave), NULL);
 #if 0
     this signal looks bugged in gtk.only works well forsame widget.segv otherwise g_signal_connect(G_OBJECT(treeview), "drag_data_delete", G_CALLBACK(on_drag_data_delete), NULL);
 #endif
@@ -1008,12 +1148,20 @@ void setup_drag_signal(GtkTreeView * treeview)
 }
 
 
-void drag_source_set(GtkTreeView * treeview)
+G_MODULE_EXPORT
+void drag_source_set (GtkTreeView * treeview)
 {
-    gtk_drag_source_set((GtkWidget *) treeview, GDK_BUTTON1_MASK | GDK_BUTTON2_MASK, target_table, NUM_TARGETS, GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK);
+#ifdef DEBUG
+    printf("DBG drag_source_set\n");
+#endif
+	gtk_drag_source_set((GtkWidget *) treeview, GDK_BUTTON1_MASK | GDK_BUTTON2_MASK, target_table, NUM_TARGETS, GDK_ACTION_MOVE | GDK_ACTION_COPY | GDK_ACTION_LINK);
 }
 
-void drag_source_unset(GtkTreeView * treeview)
+G_MODULE_EXPORT
+void drag_source_unset (GtkTreeView * treeview)
 {
-    gtk_drag_source_unset((GtkWidget *) treeview);
+#ifdef DEBUG
+    printf("DBG drag_source_unset\n");
+#endif
+   gtk_drag_source_unset((GtkWidget *) treeview);
 }
