@@ -59,6 +59,12 @@
 #include <X11/Xproto.h>
 #include <X11/Xatom.h>
 
+#ifdef HAVE_LIBSM
+#include <X11/SM/SMlib.h>
+#endif
+#include <libxfcegui4/libxfcegui4.h>
+
+
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
@@ -111,7 +117,69 @@ static gchar *text_types[]={
     NULL
 };
 
+extern
+SessionClient *session_client;
 
+static 
+void set_restart_command (int argc, char **argv){
+#if 10
+      int j,k;
+      static gchar **saved_rstart=NULL;
+      static gchar **rstart=NULL;
+
+      if (!argv[1]) return;
+      if (strcmp(argv[0],"xffm")!=0 && strcmp(argv[0],"xftree4")!=0) return;
+      
+      if (!saved_rstart){
+	    int rstart_argc;
+	    saved_rstart=session_client->restart_command;
+	    for (rstart_argc=0;saved_rstart[rstart_argc];rstart_argc++);
+	    rstart = (gchar **)malloc((argc+rstart_argc)*sizeof(gchar *));
+
+	    for(j=0;j<argc;j++){
+		rstart[j]=g_strdup(argv[j]);
+	    }
+	    for (k=1;j<argc+rstart_argc && session_client->restart_command[k];k++,j++){
+		rstart[j]=g_strdup(session_client->restart_command[k]);
+	    }
+	    rstart[argc+rstart_argc-1]=NULL;
+	    session_client->restart_command=rstart;
+      } else {
+	  g_free(rstart[1]);
+	  rstart[1]=g_strdup(argv[1]);
+      }
+
+      return;
+#endif
+#if 0
+#ifdef HAVE_LIBSM
+      int i=0;
+       SmProp prop1,*props[1];
+       SmPropValue *vals = g_new (SmPropValue, argc);
+       
+       props[0] = &prop1;
+
+	prop1.name = SmRestartCommand;
+	prop1.type = SmLISTofARRAY8;
+	prop1.vals = vals;
+	for (i=0;i<argc;i++){
+	    if (!argv[i]) {
+		argc=i;
+		break;
+	    }
+	    vals[i].value = argv[i];
+	    vals[i].length = strlen(argv[i]);
+	}
+	prop1.num_vals = argc;
+	
+	printf("DBG:xxx 0x%x\n",session_client);
+	printf("DBG:xxx 0x%x\n",session_client->session_connection);
+	SmcSetProperties ((SmcConn) session_client->session_connection, 1, props);
+	g_free(vals);
+#endif
+#endif
+
+}
 
 
 G_MODULE_EXPORT
@@ -154,6 +222,7 @@ void set_title (GtkTreeView * treeview, gchar **path)
     Atom atomo;
     char title[_POSIX_PATH_MAX], iconname[_POSIX_PATH_MAX], *hostname;
     char *word;
+    static gboolean interned=FALSE;
 
 
     hostname = our_host_name(treeview);
@@ -177,25 +246,44 @@ void set_title (GtkTreeView * treeview, gchar **path)
 		    gtk_widget_get_toplevel(tree_details->window)->window, 
 		    my_utf_string(iconname));
 
-    atomo = XInternAtom(GDK_DISPLAY(), "WM_COMMAND", FALSE);
-    XChangeProperty(GDK_DISPLAY(), get_xid(treeview), atomo, XA_STRING, 8, PropModeReplace, (unsigned char *)tree_details->argv[0], strlen(tree_details->argv[0]) + 1);
+      set_restart_command(2,tree_details->argv);
+      if (interned) {
+	static char *argv[3];
+	argv[0]=tree_details->argv[0];
+	argv[1]=tree_details->argv[1];
+	argv[2]=NULL;
+	
+	XSetCommand(GDK_DISPLAY(), get_xid(treeview), argv, 2);
+      }
+      else 
+      {
+	interned=TRUE;
+	atomo = XInternAtom(GDK_DISPLAY(), "WM_CLASS", FALSE);
+	XChangeProperty(GDK_DISPLAY(), get_xid(treeview), atomo, 
+		XA_STRING, 
+		8, PropModeReplace, 
+		(unsigned char *)"xffm", strlen("xffm") + 1);
+	atomo = XInternAtom(GDK_DISPLAY(), "WM_COMMAND", FALSE);
+	XChangeProperty(GDK_DISPLAY(), get_xid(treeview), atomo, XA_STRING, 
+		8, PropModeReplace, 
+		(unsigned char *)tree_details->argv[0], strlen(tree_details->argv[0]) + 1);
 
-    if (tree_details->argv[1]) XChangeProperty(GDK_DISPLAY(), get_xid(treeview), atomo, XA_STRING,	/* GdkAtom type, */
+	if (tree_details->argv[1]) {
+	    XChangeProperty(GDK_DISPLAY(), get_xid(treeview), 
+		    atomo, XA_STRING,	/* GdkAtom type, */
 		    8,		/* bit per data element: gint format */
-		    PropModeAppend, (unsigned char *)tree_details->argv[1], strlen(tree_details->argv[1])+1);
-    XChangeProperty(GDK_DISPLAY(), get_xid(treeview), atomo, XA_STRING,	/* GdkAtom type, */
+	    	    PropModeAppend, 
+		    (unsigned char *)tree_details->argv[1], strlen(tree_details->argv[1])+1);
+    
+	    XChangeProperty(GDK_DISPLAY(), get_xid(treeview), 
+		    atomo, XA_STRING,	/* GdkAtom type, */
 		    8,		/* bit per data element: gint format */
-		    PropModeAppend, (unsigned char *)"-H", 3);
-    XChangeProperty(GDK_DISPLAY(), get_xid(treeview), atomo, XA_STRING,	/* GdkAtom type, */
-		    8,		/* bit per data element: gint format */
-		    PropModeAppend, (unsigned char *)hostname, strlen(hostname) + 1);
-
-    XChangeProperty(GDK_DISPLAY(), get_xid(treeview), atomo, XA_STRING,	/* GdkAtom type, */
-		    8,		/* bit per data element: gint format */
-		    PropModeAppend, (unsigned char *)(*path), strlen(*path) + 1);
+		    PropModeAppend, 
+		    (unsigned char *)(*path), strlen(*path) + 1);
+	}
+    
+      }
     
-    atomo = XInternAtom(GDK_DISPLAY(), "WM_CLASS", FALSE);
-    XChangeProperty(GDK_DISPLAY(), get_xid(treeview), atomo, XA_STRING, 8, PropModeReplace, (unsigned char *)"xffm", strlen("xffm") + 1);
 
 }
 
