@@ -340,7 +340,12 @@ on_run_path(		char *in_cmd,
 
     if(getenv("XFFM_HOLD_XTERM") && strlen( getenv("XFFM_HOLD_XTERM"))) hold=TRUE;
     
-    if (path) name=g_path_get_basename(path); else name=g_strdup("");
+    /* why did I ever use a relative path here? Reverting cause it breaks
+     * doubleclick on .html files...
+     * if (path) name=g_path_get_basename(path); else name=g_strdup("");
+     * */
+    if (path) name=g_strdup(path); else name=g_strdup("");
+    
     command = MIME_mk_command_line(in_cmd, name, interm, hold);
     if (!command){
     	if (name) g_free(name);   
