@@ -63,16 +63,11 @@
 #include "widgets.h"
 
 /*#define DEBUG*/
+#define GNOME_TRASH 	g_get_home_dir(),".Trash"
+/* XXX this is kde2. I dunno about kde3*/
+#define KDE_TRASH 	g_get_home_dir(),"Desktop","Trash"
 
 
-enum
-{
-    XFCE_TRASH,
-    KDE_TRASH,
-    GNOME_TRASH,
-    HOW_MANY_OTHER_TRASH
-};
-
 static GtkTreeIter *target_iter;
 static GtkTreeView *target_treeview;
 static unsigned int trashcount;
@@ -144,14 +139,8 @@ int fork_wait(gboolean animate){
   return TRUE;
 }
 
-static void delete_trash(DBHashTable * dbh)
-{
-    struct stat st;
-    gchar *fullpath = (gchar *)DBH_DATA(dbh);
-    if(lstat(fullpath, &st) < 0)
-	return;
-    if(S_ISDIR(st.st_mode))
-    {
+static
+void rm_directory (gchar *fullpath){
 	char *argument[4];
 	argument[0]="rm";
 	argument[1]="-rfv";
@@ -168,11 +157,16 @@ static void delete_trash(DBHashTable * dbh)
 	 fork_wait(FALSE);
 	 print_diagnostics(NULL,_("Deleted")," ",fullpath,"\n",NULL);
 	 unset_load_wait();
-    }
-    else
-    {
-	g_warning("non-directory found in trashbin");
-    }
+}
+
+static void delete_trash(DBHashTable * dbh)
+{
+    struct stat st;
+    gchar *fullpath = (gchar *)DBH_DATA(dbh);
+    if(lstat(fullpath, &st) < 0)
+	return;
+    if(S_ISDIR(st.st_mode)) rm_directory(fullpath);
+    else g_warning("non-directory found in trashbin: %s",fullpath);
     return;
 }
 
@@ -233,6 +227,17 @@ delete_all_trash(GtkTreeView *treeview)
 	gtk_tree_path_free(treepath);
 	set_icon(treemodel, &iter);
     }
+/* zap gnome and kde trash too */
+    {
+	gchar *gnome=g_build_filename(GNOME_TRASH,NULL);
+	gchar *kde=g_build_filename(KDE_TRASH,NULL);
+	if (kde && g_file_test(kde,G_FILE_TEST_IS_DIR)) rm_directory(kde);
+	if (gnome && g_file_test(gnome,G_FILE_TEST_IS_DIR)) rm_directory(gnome);
+	g_free(kde);
+	g_free(gnome);
+    }
+
+    
     cursor_reset();
     print_status("xfce/info", _("Trash has been deleted"), NULL);
     unset_load_wait();
@@ -318,21 +323,57 @@ static void add_bin(DBHashTable * dbh)
 G_MODULE_EXPORT
 int open_trash(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath, gpointer user_data)
 {
+    GtkTreeIter child_iter;
+    tree_entry_t *child_en;    
     tree_entry_t *en;
+    gchar *gnome=NULL,*kde=NULL;
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
     gtk_tree_model_get(treemodel, iter, ENTRY_COLUMN, &en, -1);
     if (!en) return -1;   
     target_iter = iter;
     target_treeview = treeview;
     target_type = en->type;
+    UNSET_ERASED_SPACE(en->type);
 
     smallcount = 0, countbyte = 0x10;	/* for dummy tag */
     trashcount = 0;		/* for count step */
     trash_xfdir.pathc = 0;	/* for read step */
-    if ((trashbin=open_trash_dbh(FALSE))==NULL) return -1;
+    if ((trashbin=open_trash_dbh(FALSE))==NULL) {
+	print_diagnostics("xfce/info",_("No trash has been collected."),NULL);
+    }
 
     cursor_wait();
-
+    /* other trash */
+    gnome=g_build_filename(GNOME_TRASH,NULL);
+    kde=g_build_filename(KDE_TRASH,NULL);
+	
+    if (g_file_test(gnome,G_FILE_TEST_IS_DIR)){
+	    child_en=stat_entry(gnome, target_type);
+	    add_row (treemodel,iter, NULL, &child_iter, child_en, "[GNOME]");
+	    erase_dummy_row(treemodel, iter, NULL);
+    } else { g_free(gnome); gnome=NULL; }
+    
+    if (g_file_test(kde,G_FILE_TEST_IS_DIR)){
+	    child_en=stat_entry(kde, target_type);
+	    add_row (treemodel,iter, NULL, &child_iter, child_en, "[KDE]");
+	   erase_dummy_row(treemodel, iter, NULL);
+    } else { g_free(kde); kde=NULL; }
+    
+    if (kde || gnome) {
+	    g_free(gnome); gnome=NULL;
+	    g_free(kde); kde=NULL;
+	    erase_dummy_row(treemodel, iter, NULL);
+    } else reset_dummy_row(treemodel, iter,NULL,en,NULL,NULL);
+    
+    if (!trashbin) {
+	SET_LOADED(en->type);
+	cursor_reset();
+	return 0;
+    }
+    
+    /*child_en = mk_entry(en->type);
+    add_row (treemodel,iter, NULL, &child_iter, child_en, "[Xfce]");*/
+    
     just_count = TRUE;
     DBH_foreach_sweep(trashbin, add_bin);
 #ifdef DEBUG
@@ -344,44 +385,36 @@ int open_trash(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepat
 	trash_xfdir.gl = (dir_t *) malloc(trashcount * sizeof(dir_t));
 	if(!trash_xfdir.gl)
 	{
-	    DBH_close(trashbin);
+	    if (trashbin) DBH_close(trashbin);
+	    g_warning("malloc(trashcount * sizeof(dir_t)) == NULL");
 	    return -1;
 	}
 	just_count = FALSE;
 	DBH_foreach_sweep(trashbin, add_bin);
-#ifdef DEBUG
-        printf("DBG:pathc=%d\n",trash_xfdir.pathc); 
-	printf("DBG:count=%d\n",trashcount); 
-#endif
+        D(printf("DBG:pathc=%d\n",trash_xfdir.pathc); )
+	D(printf("DBG:count=%d\n",trashcount); )
 	if(trashcount != trash_xfdir.pathc)
 	{
 	    SET_ERASED_SPACE(en->type);
-#ifdef DEBUG
-	    printf ("DBG:listed=%d, counted=%d\n", trash_xfdir.pathc,
-	       trashcount); 
-#endif
+	    D(printf ("DBG:listed=%d, counted=%d\n", trash_xfdir.pathc, trashcount); )
 	}
 
 
 	add_contents_row(treemodel, iter, &trash_xfdir);
-	erase_dummy_row(treemodel, iter, NULL);
 	xfdirfree(&trash_xfdir);
-    }
-    else
-    {
-	SET_LOADED(en->type);
-	reset_dummy_row(treemodel, iter,NULL,en,NULL,NULL);
-    }
+    } 
+    /*else { remove_row(treemodel,&child_iter, NULL,child_en);}*/
+    
+    SET_LOADED(en->type);
+    /*if (trashcount) erase_dummy_row(treemodel, iter, NULL);*/
+    /*else reset_dummy_row(treemodel, iter,NULL,en,NULL,NULL);*/
+    
     if(DBH_ERASED_SPACE(trashbin))
     {
-#ifdef DEBUG
-	printf("DBG: erased space=%d\n",DBH_ERASED_SPACE(trashbin)); 
-#endif
-	
+	D(printf("DBG: erased space=%d\n",DBH_ERASED_SPACE(trashbin)); )
 	SET_ERASED_SPACE(en->type);
     }
-    else
-	UNSET_ERASED_SPACE(en->type);
+    /*(else UNSET_ERASED_SPACE(en->type);*/
     DBH_close(trashbin);
     if (en->tag){
 	    g_free(en->tag);
@@ -400,10 +433,7 @@ int open_trash(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepat
 	hide_stop();
 	en->tag = g_strdup_printf("%s : %d %s", _("Trash"), trash_xfdir.pathc, _("items"));
     }
-
-    /*gtk_widget_thaw_child_notify  ((GtkWidget *)treeview);*/
     cursor_reset();
-
     return 0;
 }
 
@@ -415,10 +445,8 @@ static int read_trash_locations(char *path)
     int count = 0;
     struct dirent *d;
     regex_t shortpreg;
-    regex_t fullpreg;
     gchar *newpath=NULL;
-    gchar *ot;
-    gboolean shortpreg_compiled,fullpreg_compiled;
+    gboolean shortpreg_compiled;
 
     if(tree_details->stop) return 0;
     set_progress(-1, 0);
@@ -426,22 +454,13 @@ static int read_trash_locations(char *path)
     directory = opendir(path);
     if(!directory)
 	return 0;
-
-    /* XFCE3 trash: */
-    ot = g_strconcat("^",GETWD,G_DIR_SEPARATOR_S,".xfce/trash$",
-    /* KDE trash: */
-			"|^",GETWD,G_DIR_SEPARATOR_S,"Desktop",G_DIR_SEPARATOR_S,"Trash$",
-    /* gnome trash: */
-			"|^",GETWD,G_DIR_SEPARATOR_S,"\\.Trash$",NULL);
-
-
-    if (regcomp(&fullpreg, ot, REG_EXTENDED | REG_ICASE | REG_NOSUB)==0) fullpreg_compiled=TRUE;
-    else fullpreg_compiled=FALSE;
-    g_free(ot); 
-    if (regcomp(&shortpreg, "^\\.\\.Wastebasket$", REG_EXTENDED | REG_ICASE | REG_NOSUB)) 
+    
+    if (!regcomp(&shortpreg, "^\\.\\.Wastebasket$", REG_EXTENDED | REG_ICASE | REG_NOSUB)) 
 	shortpreg_compiled=TRUE;
     else 
 	shortpreg_compiled=FALSE;
+
+    
     while((d = readdir(directory)) != NULL)
     {
 	if(tree_details->stop)
@@ -454,8 +473,7 @@ static int read_trash_locations(char *path)
 	set_progress(-1, 0);
 
 	newpath=g_build_filename(path, d->d_name, NULL);
-	if((fullpreg_compiled && !regexec(&fullpreg, newpath, 0, NULL, 0)) || 
-	    (shortpreg_compiled && !regexec(&shortpreg, d->d_name, 0, NULL, 0)) )
+	if(shortpreg_compiled && !regexec(&shortpreg, d->d_name, 0, NULL, 0) )
 	{
 	    GString *gs;
 #ifdef DEBUG
@@ -486,7 +504,6 @@ static int read_trash_locations(char *path)
     }
     closedir(directory);
     if (shortpreg_compiled)regfree(&shortpreg);
-    if (fullpreg_compiled) regfree(&fullpreg);
     return (count);
 }
 
