@@ -1,7 +1,6 @@
-/*#define DEBUG*/
 /*
- * Copyright (C) 2002 Edscott Wilson Garcia
- * EMail: edscott@imp.mx
+ * Copyright (C) 2002-4 Edscott Wilson Garcia
+ * EMail: edscott@xfce.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,54 +29,60 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <grp.h>
+#include <pwd.h>
 
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
-#include "glade_callbacks.h"
-#include "glade_gui.h"
 #include "glade_support.h"
 
 #include "constants.h"
 #include "types.h"
+#include "modules.h"
 
-#include "apps.h"
-#include "add_file.h"
-#include "add_folder.h"
-#include "add_node_contents.h"
-#include "aux.h"
+#include "basic_row.h"
+
+#include "callbacks.h"
+
+#include "basic_local.h"
 #include "bookmarks.h"
 #include "dnd.h"
-#include "dummies.h"
 #include "entry.h"
+
+#ifdef USE_FILTER_BAR
 #include "filter.h"
+#endif
+
 #include "goto.h"
 #include "icons.h"
 #include "input.h"
+#include "ls.h"
 #include "menu.h"
 #include "misc.h"
 #include "monitor.h"
-#include "reg.h"
 #include "run.h"
-#include "smb_misc.h"
-#include "smb_open.h"
+#include "password_dialog.h"
 #include "trash.h"
 #include "treeview.h"
 #include "treestore.h"
 #include "widgets.h"
 #include "xfstab.h"
-
+#include "keybindings.h"
+#include "recent.h"
 
 /* bug workaround (double click signals click first)*/
-static gboolean skip = FALSE;
+static gboolean skip_second_release = FALSE;
 static GtkTreeRowReference *title_reference=NULL;
-static mousedown=FALSE;
-static gboolean unsel(GtkTreeModel *,GtkTreePath *, GtkTreeIter *, gpointer);
+static gboolean mousedown=FALSE;
+static gdouble mouseX,mouseY;
+static GtkTreePath *clickpath;
+G_MODULE_EXPORT
+gboolean easy_mode=FALSE;
 
-extern int stop;
-/*XXX for gtk bug workaround with flashing treeview on expand/collapse
- * ater scroll if root not visible: */
-extern GtkTreeViewColumn *name_column;
+
+
+static gboolean unsel(GtkTreeModel * treemodel, GtkTreePath * treepath, GtkTreeIter * iter, gpointer data);
 
 static gboolean valid_iter(GtkTreeView * treeview,GtkTreeIter * iter){
     tree_entry_t *en;
@@ -88,24 +93,28 @@ static gboolean valid_iter(GtkTreeView * treeview,GtkTreeIter * iter){
     return TRUE;
 }
 
+extern gboolean no_sorting;/* see explanation in treeview.c */
+G_MODULE_EXPORT
 void open_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath, gpointer user_data)
 {
     tree_entry_t *en;
     struct stat st;
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
-    tree_details_t *tree_details = get_tree_details(treeview);
-    char *filter = get_filter(tree_details->window);
     GtkTreeIter child;
     static gchar **window_title=NULL;
     static gboolean red_light=FALSE;
-
+    GtkTreeSortable *sortable=GTK_TREE_SORTABLE(treemodel);
+    
 #ifdef DEBUG
 	printf("DBG: open_dir ...\n");
 #endif
     mousedown=FALSE; /* gtk bug workaround:click on expander passes button_press but not button_release */
+    if (clickpath){
+        clear_path_from_selection_list(treeview,clickpath);
+    }
 
     if (!valid_iter(treeview,iter)){
-	    local_monitor(treeview,TRUE);
+	    local_monitor(TRUE);
 	    return;
     }
    if (!window_title) {
@@ -113,7 +122,7 @@ void open_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath
 	    *window_title=NULL;
     }
   
-    if(!set_load_wait(&tree_details))
+    if(!set_load_wait())
     {
 	gtk_tree_view_collapse_row(treeview, treepath);
 	return;
@@ -124,25 +133,31 @@ void open_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath
 #endif
 	return;
     }
-    red_light=TRUE;	    
-    cursor_wait(treeview);
+    red_light=TRUE;
+/* disable internal sorting while loading... **/
+    no_sorting=TRUE;
+	
+    cursor_wait();
     gtk_tree_model_get(GTK_TREE_MODEL(treemodel), iter, ENTRY_COLUMN, &en, -1);
 #ifdef DEBUG
 	printf("DBG: open_dir callback...\n");
 #endif
     if(!en || !en->path) goto time2return;
     if (!IS_FSTAB_TYPE(en->type) && 
-	!IS_APP_TYPE(en->type) && 
 	!IS_TRASH_TYPE(en->type) && 
 	!IS_NETWORK_TYPE(en->type) && 
 	!IS_BOOKMARK_TYPE(en->type) && 
+	!IS_RECENT_TYPE(en->type) && 
+	!IS_FREQUENT_TYPE(en->type) && 
 	!en->st
        ) goto time2return;
 
+#ifdef USE_FILTER_BAR
     if(en->filter && strcmp(en->filter, get_filter(tree_details->window))){
-	/*printf("DBG: UNSET_LOADED filter && strcmp(filter, get_filter)\n");*/
+	/*printf("DBG: UNSET_LOADED filter && strcmp(filter, get_filter) filter=%s get_filter=%s\n",en->filter,get_filter(tree_details->window));*/
 	UNSET_LOADED(en->type);
     }
+#endif
 
     if(stat(en->path, &st) >= 0)
     {
@@ -152,6 +167,15 @@ void open_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath
 	    memcpy(en->st, &st, sizeof(struct stat));
 	    UNSET_LOADED(en->type);
 	}
+        if(access(en->path, X_OK)!=0){ /* abort the expand... */
+	   reset_dummy_row(treemodel, iter,NULL,en,"xfce/error",strerror(errno));
+    	   hide_stop();
+	   unset_load_wait();
+	   cursor_reset();
+	   red_light=FALSE;
+	   return;
+	}
+	
     }
     else if(IS_PATH(en->type) && lstat(en->path, &st) < 0)
     {
@@ -169,23 +193,21 @@ void open_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath
     if(IS_ROOT_TYPE(en->type) && 
        (IS_BOOKMARK_TYPE(en->type) || 
 	IS_TRASH_TYPE(en->type) || 
-	IS_APP_TYPE(en->type) || 
+	IS_RECENT_TYPE(en->type) || 
+	IS_FREQUENT_TYPE(en->type) || 
 	IS_FSTAB_TYPE(en->type)
        )
       )
     {
 	if(!IS_LOADED(en->type))
 	{
-	    if(IS_APP_TYPE(en->type))
+            if(IS_RECENT_TYPE(en->type) || IS_FREQUENT_TYPE(en->type))
 	    {
-		/*while(gtk_events_pending()) gtk_main_iteration();*/
-		if(open_apps(treeview, iter, treepath, user_data) <= 0)
+		if(open_history(treeview, iter, treepath, user_data) < 0)
 		{
-		    print_status(treeview, "xf_ERROR_ICON", 
+		    print_status("xfce/error", 
 				    strerror(errno),NULL);
-		    reset_dummy(treeview, iter, -8);
 		}
-    		/*hide_stop(tree_details->window);*/
 	        goto time2return;
 	    }
             if(IS_FSTAB_TYPE(en->type))
@@ -193,68 +215,48 @@ void open_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath
 		/*while(gtk_events_pending()) gtk_main_iteration();*/
 		if(open_fstab(treeview, iter, treepath, user_data) < 0)
 		{
-		    print_status(treeview, "xf_ERROR_ICON", 
+		    print_status("xfce/error", 
 				    strerror(errno),NULL);
 		}
-    		/*hide_stop(tree_details->window);*/
+    		/*hide_stop();*/
 	        goto time2return;
 	    }
 	    if(IS_BOOKMARK_TYPE(en->type))
 	    {
-		print_status(treeview, "xf_WARNING_ICON", 
-				_("Loading bookmarks"), NULL);
-		/*while(gtk_events_pending()) gtk_main_iteration();*/
 		if(open_bookmarks(treeview, iter, treepath, user_data) < 0)
 		{
-		    print_status_tmp(treeview, resolve_icon_small(en), 
-				    _("No bookmarks have been collected."), 
-				    NULL);
-		    reset_dummy(treeview, iter, 3);
+		    reset_dummy_row(treemodel, iter, NULL,en,"xfce/info",_("Use drag+drop to add"));
 		}
-		if (en->tag && strchr(en->tag,'%'))
-		   print_status(treeview,  "xf_BOOKMARKS_ICON",
-			       	" ", NULL);
-		else
-		   print_status(treeview, "xf_BOOKMARKS_ICON", 
-				   en->tag, NULL);
+		print_status(NULL,"", NULL);
 	        goto time2return;
 	    }
 	    if(IS_TRASH_TYPE(en->type))
 	    {
 		int result;
 		gtk_tree_view_collapse_row(treeview, treepath);
-		print_status(treeview, "xf_WARNING_ICON", 
-				"Loading trash bin", NULL);
-		/*while(gtk_events_pending()) gtk_main_iteration();*/
 		if((result = open_trash(treeview, iter, treepath, user_data)) < 0)
 		{
 
-		    print_status_tmp(treeview, resolve_icon_small(en), 
-				    (result == -1) ?
-				    _("No trash has been collected.") :
-				    _("Trash load aborted."), NULL);
-		    /*reset_dummy (treeview, iter, 1); */
-		    /*hide_stop(tree_details->window);*/
+		    if (result == -2) print_status_tmp(resolve_icon_small(en), 
+			    _("Trash load aborted."), NULL);
+		    else {
+			print_status(NULL,"",NULL);
+			reset_dummy_row (treemodel, iter, NULL,en,"xfce/info", _("No trash has been collected.")); 
+		    }
 		}
 		else
 		{
-		    /*while(gtk_events_pending()) gtk_main_iteration();*/
-		    unset_load_wait(&tree_details);
-		    hide_stop(tree_details->window);
-		    print_status(treeview, "xf_WARNING_ICON", "Expanding trash bin", NULL);
-		    gtk_tree_view_expand_row(treeview, treepath, FALSE);
+		    hide_stop();
+		    print_status(NULL,"",NULL);
 		}
+		unset_load_wait();
+		gtk_tree_view_expand_row(treeview, treepath, FALSE);
 	    }
 	    goto time2return;
 	}
-	if (en->tag && strchr(en->tag,'%'))
-		print_status_tmp(treeview, resolve_icon_small(en),
-			       	FILENAME(en), NULL);
-	else
-		print_status_tmp(treeview, resolve_icon_small(en),
-			       	en->tag, NULL);
+	print_status(NULL,"", NULL);
 	/*set_select_path(treemodel,iter,tree_details); */
-	/* when expand icon is different: set_icon(treeview,iter); */
+	/* when expand icon is different: set_icon(treemodel, iter); */
 
     }
     if(!IS_LOADED(en->type) && 
@@ -265,83 +267,26 @@ void open_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath
       )
     {
 	    /*printf("DBG: network...\n");*/
-    process_pending_gtk();
-
-	if(!open_smb(treeview, iter, treepath, user_data))
+	process_pending_gtk();
+#ifdef USE_SMB_BRANCH
+	if(!XF_open_smb(treeview, iter, treepath, user_data))
 	{
-	    /*print_status(treeview, "xf_ERROR_ICON", strerror(errno),NULL);*/
 	}
-    	/*hide_stop(tree_details->window);*/
+#endif
         goto time2return;
     }
     if(IS_NETWORK_TYPE(en->type) && IS_LOADED(en->type)){
 	SET_EXPANDED(en->type);
-	set_icon(treeview, iter);
     }
-    if(IS_APP_TYPE(en->type)){
-	    reset_dummy(treeview, iter, 1);	    
+    if(IS_DIR(en->type)) {
+	open_folder_local(treeview,iter,treepath,en);
     }
-    if(IS_DIR(en->type))
-    {
-	/*printf("DBG:opening %s\n",en->path); */
-	/* if filterchanged reload as well */
-	if(!IS_LOADED(en->type) || IS_INCOMPLETE(en->type) ||
-	   (IS_LOADED(en->type) &&
-    	    ((!filter && en->filter)  || 
-	      (filter && !en->filter) || 
-	      strcmp(en->filter, filter)
-	     )
-	    )
-	   )
-	{
-	    /*printf("DBG: loaded=%d, filter=%s,en->filter=%s\n",
-			    IS_LOADED(en->type),
-			    (filter)?filter:"NULL",
-			    (en->filter)?en->filter:"NULL");*/
-	    remove_folder(treeview, iter);
-	    UNSET_INCOMPLETE(en->type);
-	    print_status(treeview, "xf_WARNING_ICON", "Loading", " : ", FILENAME(en), NULL);
-    process_pending_gtk();
 
-	    if(access(en->path, X_OK))
-	    {
-		goto time2return;
-	    }
-	    add_folder(treeview, iter);
-	    gtk_tree_store_set((GtkTreeStore *) treemodel, iter, SIZE_COLUMN, sizetag(-1, en->count), -1);
-
-	    unset_load_wait(&tree_details);
-	    hide_stop(tree_details->window);
-	    if (IS_INCOMPLETE(en->type))
-		    print_status(treeview,"xf_WARNING_ICON",
-				    _("Load is incomplete"),NULL);
-	    else SET_LOADED(en->type);
-	    /*printf("DBG: callback expanding\n");*/
-	    gtk_tree_view_expand_row(treeview, treepath, FALSE);
-	}
-	if(!tree_details->window)  exit(1);
-	/*bug causer here : while(gtk_events_pending())  gtk_main_iteration();*/
-        /*this might not cause a bug because a race condition with
-	 * gtk treeview operations, but YMMV */
-	/*process_pending_gtk();*/
-
-	/*set_select_path(treemodel,iter,tree_details); */
-	set_icon(treeview, iter);
-
-	    /*printf("DBG: tag writing %s\n",en->path);*/
-	if (en->tag && strchr(en->tag,'%'))
-		print_status_tmp(treeview, resolve_icon_small(en),
-			       	FILENAME(en), NULL);
-	else
-		print_status_tmp(treeview, resolve_icon_small(en),
-			       	en->tag, NULL);
-
-    }
-  time2return:
+time2return:
 #ifdef DEBUG
 	printf("DBG: opendir time2return.\n"); 
 #endif
-    set_icon(treeview,iter);
+    set_icon(treemodel, iter);
 	
 #if 10
     if(gtk_tree_model_iter_children(treemodel, &child, iter)){
@@ -355,21 +300,23 @@ void open_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath
 #ifdef DEBUG
 	      printf("DBG: icon setting for %s\n",c_en->path); 
 #endif
-		set_icon(treeview, &child);
+		set_icon(treemodel, &child);
 	}
       } while(gtk_tree_model_iter_next(treemodel, &child));
     }
 #endif
-    hide_stop(tree_details->window);
+    
+    hide_stop();
     if(!tree_details->window) exit(1);
 
 #ifdef DEBUG
 	printf("DBG: open_dir, unsetting load wait\n"); 
 #endif
-    unset_load_wait(&tree_details);
-   /* bug causer here : while(gtk_events_pending())gtk_main_iteration();*/
-   /* and even this will ocasional do a crash: process_pending_gtk(); 
-    * it is cause by a race condition with gtk internals...*/
+    unset_load_wait();
+
+    if (!gtk_tree_view_row_expanded(treeview,treepath)) {
+	    gtk_tree_view_expand_row(treeview, treepath, FALSE);
+    }
 #ifdef DEBUG
 	printf("DBG: open_dir, setting title\n"); 
 #endif
@@ -386,7 +333,6 @@ void open_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath
 	} 
 	set_title(treeview,window_title);
     }
-
     if(tree_details->preferences & AUTOSCROLL){
 	gdk_flush();
 	/*XXX gtk bug, scrolling does not always occur...
@@ -395,33 +341,45 @@ void open_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * treepath
 	gtk_tree_view_scroll_to_cell(treeview, treepath, NULL, TRUE, 0.0, 0.0);
     }
 
-    cursor_reset(treeview);
+    cursor_reset();
     
-    /*XXX gtk bug workaround */
-    if (!IS_ROOT_TYPE(en->type)){
-        gtk_tree_view_column_set_resizable(name_column, TRUE);
-    }
    
     red_light=FALSE;
+    /*{
+      GdkRectangle rect;
+      rect.x=rect.y=0;
+      rect.width=tree_details->window->allocation.width;
+      rect.height=tree_details->window->allocation.height;
+      gdk_window_invalidate_rect(gtk_widget_get_toplevel(tree_details->window)->window,&rect,TRUE);
+      gdk_window_process_updates(gtk_widget_get_toplevel(tree_details->window)->window,TRUE);
+    }*/
+    
 #ifdef DEBUG
 	printf("DBG: open_dir, all done!\n"); 
 #endif
-   
+    unset_load_wait();
+     /* reenable internal sorting now... */
+    no_sorting=FALSE;
+    gtk_tree_sortable_sort_column_changed (sortable);
+  
 }
 
 
+G_MODULE_EXPORT
 void close_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * path, gpointer user_data)
 {
     tree_entry_t *en, *p_en;
     GtkTreeIter parent,child;
     static gchar **window_title=NULL;
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
-    tree_details_t *tree_details = get_tree_details(treeview);
 
     mousedown=FALSE; /* gtk bug workaround:click on expander passes button_press but not button_release */
+    if (clickpath){
+        clear_path_from_selection_list(treeview,clickpath);
+    }
 
     if (!valid_iter(treeview,iter)){
-	    local_monitor(treeview,TRUE);
+	    local_monitor(TRUE);
 	    return;
     }
     if(tree_details->loading)
@@ -437,11 +395,8 @@ void close_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * path, g
 
     gtk_tree_model_get(treemodel, iter, ENTRY_COLUMN, &en, -1);
     if(!en) goto time2return;
-    /*XXX gtk bug workaround */
-    if (IS_ROOT_TYPE(en->type)){
-        gtk_tree_view_column_set_resizable(name_column, TRUE);
-    	gtk_tree_view_column_set_sizing(name_column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-    }
+
+
     UNSET_EXPANDED(en->type);
     if(gtk_tree_model_iter_children(treemodel, &child, iter)){
       do {
@@ -451,90 +406,96 @@ void close_dir(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * path, g
       } while(gtk_tree_model_iter_next(treemodel, &child));
     }
 
-    set_icon(treeview, iter);
+    set_icon(treemodel, iter);
     
-#if 0
-    if (IS_ROOT_TYPE(en->type) &&
-	(IS_APP_TYPE(en->type)||IS_BOOKMARK_TYPE(en->type))
-       )
-    {
-      remove_folder(treeview, iter);
-    }
-#endif
     
    if(IS_TRASH_TYPE(en->type))
     {
-#if 0
-	if(en->count > 512)
-	{
-	    remove_folder(treeview, iter);
-	}
-#endif
 	/*set_select_path(treemodel,iter,tree_details); */
-	print_status_tmp(treeview, resolve_icon_small(en), en->tag, NULL);
-
+	print_status(NULL,"", NULL);
     }
-    /*if(IS_LOCAL_TYPE(en->type))*/
     if(en->path)
     {
 	g_free(*window_title);
 	*window_title=NULL;
-#if 0
-	if(en->count > 512)
-	{
-	    /*printf("removing %s count=%u\n",en->path,en->count); */
-	    remove_folder(treeview, iter);
-	}
-#endif
 	if(gtk_tree_model_iter_parent(treemodel, &parent, iter))
 	{
 	    GtkTreePath *tpath=gtk_tree_path_copy(path);
 	    gtk_tree_path_up(tpath);
 	    gtk_tree_model_get(treemodel, &parent, ENTRY_COLUMN, &p_en, -1);
-	    /*set_select_path(treemodel,&parent,tree_details); */
 	    *window_title=g_strdup(p_en->path);
     	    if (title_reference) gtk_tree_row_reference_free(title_reference);
        	    title_reference = gtk_tree_row_reference_new(treemodel, tpath);
-	    /*FIXME This might segv on race condition: */
-	    print_status(treeview, resolve_folder_icon(p_en), p_en->tag, NULL);
+	    if (p_en->tag) {
+	     if (strchr(p_en->tag,'%')){	 
+	      char *p=g_strdup(en->tag);
+	      p=strtok(p,"%");
+	      print_status((char *)resolve_folder_icon(p_en),p, NULL);
+	      g_free(p);
+	     } else print_status((char *)resolve_folder_icon(p_en), p_en->tag, NULL);
+	    }
+	    else  print_status(NULL,"", NULL);
 	    set_path_reference(treeview,tpath);
 	    gtk_tree_path_free(tpath);
 	}
 	else
 	{
-	    /*set_select_path(treemodel,iter,tree_details); */
 	    *window_title=g_strdup(en->path);
     	    if (title_reference) gtk_tree_row_reference_free(title_reference);
        	    title_reference = gtk_tree_row_reference_new(treemodel, path);
-	    print_status(treeview, resolve_folder_icon(en), en->tag, NULL);
+	    if (en->tag) {
+   	     if (strchr(en->tag,'%')){	 
+	      char *p=g_strdup(en->tag);
+	      p=strtok(p,"%");
+	      print_status((char *)resolve_folder_icon(en),p, NULL);
+	      g_free(p);
+	     } else print_status((char *)resolve_folder_icon(en), en->tag, NULL);
+	    }
+	    else  print_status(NULL,"", NULL);
 	}
 	set_title(treeview,window_title);
     }
   time2return:
-    turn_on(treeview);
+    turn_on();
     tree_details->loading = FALSE;
 }
 
 
 /********************* callbacks **************************/
 
-gboolean on_mouse_move(GtkWidget *widget,GdkEventMotion *event, gpointer user_data)
-{
-	static gboolean reselected=FALSE;
-	if (!mousedown) {
-		reselected=FALSE;
-	}
-	else if (!reselected) {
-		reselected=TRUE;
- 		reselect_dnd_list((GtkTreeView *)widget);
-	}
+static int reselecttimer=0;
+static gint do_reselect(gpointer data){
+	GtkTreeView * treeview=(GtkTreeView * )data;
+ 	reselect_dnd_list(treeview);
+	/*printf("DBG:at reselecttimer = %d\n",reselecttimer );*/
+	reselecttimer=0;
 	return FALSE;
 }
-	
+
+static gint delayed_selection(gpointer data){
+	GtkTreeView * treeview=(GtkTreeView * )data;
+        GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
+#ifdef DEBUG
+	 printf("delayed selection...\n");
+#endif
+     	gtk_tree_selection_selected_foreach(selection, get_dnd_selection, (gpointer) treeview);
+#if DEBUG_IN_DND
+	/* DEBUG must be enabled in dnd.c for this to work: */
+#ifdef DEBUG
+	{
+	    extern void the_sl(void);
+	    the_sl();
+	}
+#endif
+#endif
+	return FALSE;
+}
+
+
+G_MODULE_EXPORT
 gboolean button_releaseF(GtkWidget * widget, GdkEventButton * event, gpointer data)
 {
     GtkTreeView *treeview = (GtkTreeView *) data;
-    tree_details_t *tree_details = get_tree_details(treeview);
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
     GtkTreePath *treepath;
     tree_entry_t *en;
@@ -542,12 +503,24 @@ gboolean button_releaseF(GtkWidget * widget, GdkEventButton * event, gpointer da
 
    mousedown=FALSE; 
     if(tree_details->loading)	return FALSE;
+    mouseX=event->x;
+    mouseY=event->y;
+    { 
+        int i;
+        for (i=0;i<TREECOUNT;i++) {
+	   gtk_tree_view_set_drag_dest_row (tree_details->treestuff[i].treeview,NULL,GTK_TREE_VIEW_DROP_INTO_OR_BEFORE);
+	}
+    }
 
-   if(event->button == 1 && skip) 
+   if(event->button == 1 && skip_second_release) 
     {
-	skip = FALSE;
+	skip_second_release = FALSE;
+	/*printf("skip_second_release = 1\n");*/
 	return FALSE;
     }
+    /* this corrupts the double click because the treeview gets moved:
+     *  hide_text(widget);
+     *  */
     switch (event->button)
     {
 	default:
@@ -555,132 +528,169 @@ gboolean button_releaseF(GtkWidget * widget, GdkEventButton * event, gpointer da
 	case 4:
 	case 5: /* these don't work ... */
 	    break;
-	case 3:
-	    break;		/* not working for button 3, because of popup AFAICG */
-	case 1:
+	case 1: case 3:
+	if (event->button == 3)
+	    break;		/* not working for button 3 */
+	/* BUTTON_1: */
 	    if(gtk_tree_view_get_path_at_pos(treeview, event->x, event->y, 
 				    &treepath, NULL, NULL, NULL))
 	    {
-		gchar *readable_path;
+#if 0		
 		gtk_tree_model_get_iter(treemodel, &iter, treepath);
-
 		gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
-		readable_path=g_strdup(FILENAME(en));
-		if (IS_NETWORK_TYPE(en->type)&&!IS_SAMBA_SERVER(en->subtype))
+		update_status_line(en);
+		if (!en || !en->path){
+			print_status_tmp(NULL, "",NULL);
+	        } else {
+		  gchar *readable_path=g_path_get_basename(en->path);
+		  if (IS_NETWORK_TYPE(en->type)&&!IS_SAMBA_SERVER(en->subtype))
 			ascii_readable(readable_path);
-		print_status_tmp(treeview, resolve_icon_small(en), 
+		  print_status_tmp(resolve_icon_small(en), 
 				readable_path,
 				NULL);
-		g_free(readable_path);
-		readable_path=NULL;
-		set_path_reference(treeview,treepath);
-		
-		gtk_tree_path_free(treepath);
-		turn_on(treeview);
+		  g_free(readable_path);
+		  set_path_reference(treeview,treepath);
+		}
+#endif		
+	        gtk_tree_path_free(treepath);
+	        turn_on();
 	    }
 	    clear_dnd_selection_list();
-	    break;
+	    break; /* case 1 */
 	case 2:
 	    if(gtk_tree_view_get_path_at_pos(treeview, event->x, event->y, &treepath, NULL, NULL, NULL))
 	    {
 		gtk_tree_model_get_iter(treemodel, &iter, treepath);
 		gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
+		/* This is user%password for smb stuff: 
+		 * printf("DBG: tag=%s\n",en->tag);*/
 		/* trash and find entries always printout full path somewhere */
 		if( en->path && 
 		    (IS_LOCAL_TYPE(en->type) || !IS_ROOT_TYPE(en->type)) )
 		{
+		    const gchar *typeinfo=NULL;
 		    gchar *readable_path;
-		    char *icono = NULL;
-		    if(IS_APP_TYPE(en->type) && en->filter){
-			    readable_path=(gchar *)malloc(strlen(en->path)+strlen(en->filter)+2);
-			    sprintf(readable_path,"%s %s",en->path,en->filter);
-		    }
-		    else readable_path=g_strdup(en->path);
-		    if(IS_FIND_TYPE(en->type))
-			icono = "xf_FIND_RESULT_ICON";
-		    else if(IS_TRASH_TYPE(en->type))
-			icono = "xf_TRASH_OPEN_ICON";
+		    const gchar *mimetype = NULL;
+		    
+		    readable_path=g_strdup(en->path);
 		    if (IS_NETWORK_TYPE(en->type)&&!IS_SAMBA_SERVER(en->subtype)){
 			ascii_readable(readable_path);
-		    } 
-		    print_diagnostics(treeview, icono, 
-				readable_path, "\n", NULL);
+		    }
+		  
+		    if (IS_PATH(en->type) ){
+			    mimetype = MIME_get_type(en->path,TRUE);
+			    typeinfo = MIME_typeinfo(mimetype);  
+		    } else {
+			    mimetype = resolve_icon_id(en);
+			    if (!mimetype) mimetype = MIME_get_type(en->path,FALSE);
+		    }
+		    /* printout full file information: */
+		    
+		    if (IS_XF_LNK(en->type)){
+			char lpath[_POSIX_PATH_MAX+1];
+			memset(lpath,0,_POSIX_PATH_MAX+1);
+			if (readlink(en->path, lpath, _POSIX_PATH_MAX)>0) {
+			    print_diagnostics("xfce/info",en->path,"->",lpath,"\n", NULL);
+			} 
+		    }
+			    
+		    if (typeinfo && strcmp(typeinfo,"Unknown")!=0){
+		       print_diagnostics((char *)mimetype,readable_path," (",mimetype,": ",typeinfo,")\n",NULL);
+		    } else {
+		       print_diagnostics((char *)mimetype,readable_path," (",mimetype,")\n", NULL);
+		    }
+		    if (en->st) {
+    		      struct group *g;
+		      struct passwd *p;
+		      char *grupo,*owner;
+		      const gchar *tag = sizetag((off_t)en->st->st_size, -1);
+		      if((g = getgrgid(en->st->st_gid)) != NULL)	grupo = g->gr_name;
+		      else if ((int)en->st->st_gid < 0) grupo = ""; else grupo = "?";
+		      if((p = getpwuid(en->st->st_uid)) != NULL) owner = p->pw_name;
+		      else if ((int)en->st->st_uid < 0) owner = ""; else owner = "?";
+		      print_diagnostics("nonverbose",
+			    my_utf_string(time_to_string(en->st->st_mtime)),"; ",
+			    tag,", ", 
+			    owner,":",
+			    grupo," ",
+			    mode_string(en->st->st_mode),"\n", NULL);
+		    }
 		    g_free(readable_path);
-		    readable_path=NULL;
 		}
-
 		gtk_tree_path_free(treepath);
 	    }
 	    clear_dnd_selection_list();
-	    break;
-    }
+	    break; /* case 2 */
+    } /* end switch */
     return FALSE;
 }
 
-extern gboolean double_treeview;
-
 
+G_MODULE_EXPORT
 gboolean treeclick(GtkWidget * widget, GdkEventButton * event, gpointer data)
 {
     GtkTreeView *treeview = (GtkTreeView *) data;
     GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
-    tree_details_t *tree_details = get_tree_details(treeview);
+    treestuff_t *treestuff=get_treestuff(treeview);
     tree_entry_t *en;
     GtkTreePath *treepath;
     GtkTreeIter iter,child;
     gchar *newpath=NULL;
     int type;
+    gint tree_id;
+    gboolean double_click=FALSE;
 
     mousedown=TRUE;
+    easy_mode=FALSE;
+    /*dnd_mouse_moved=FALSE;*/
     if(tree_details->loading)
 	return TRUE;
 
     if(tree_details->input)
     {
-	cancel_input(treeview);
+	cancel_input(NULL,NULL);
     }
+    skip_second_release = FALSE;
 
-
-    if (double_treeview ){
-      if (treeview == tree_details->treeview){
+    for (tree_id=0; tree_id<TREECOUNT; tree_id++) {      
+      static gboolean unsel(GtkTreeModel *,GtkTreePath *, GtkTreeIter *, gpointer);
+      /*printf("DBG: treeview is  0x%x==0x%x\n",tree_details->treestuff[tree_id].treeview,treeview);*/
+      if (treeview != tree_details->treestuff[tree_id].treeview){
         /* unselect all in auxiliary treeview */
-	GtkTreeView *treeview2=(GtkTreeView *)lookup_widget(tree_details->window,"treeview2");
-        GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview2);
-	gtk_tree_model_foreach(treemodel,unsel,selection);
-      } else {
-        /* unselect all in main treeview */
-        GtkTreeSelection *selection = gtk_tree_view_get_selection(tree_details->treeview);
-        gtk_tree_model_foreach(treemodel,unsel,selection);
+        GtkTreeSelection *selection = gtk_tree_view_get_selection(tree_details->treestuff[tree_id].treeview);
+	gtk_tree_model_foreach(tree_details->treestuff[tree_id].treemodel,unsel,selection);
       }
     }
     
-    /*//if(event->state==GDK_CONTROL_MASK &&(event->button == 4 || event->button == 5)){
-    if((event->button == 4 || event->button == 5)){
-      printf("DBG:ctr button %d, state=%d\n",event->button,event->state);
-	return TRUE;
-    }*/
-    
-    if((event->type == GDK_2BUTTON_PRESS) && (event->button == 1) && event->state!=GDK_CONTROL_MASK)
-    {				/* this works because item is selected at first click of double click */
+    if((event->type == GDK_2BUTTON_PRESS) && (event->button == 1) && !(event->state & GDK_CONTROL_MASK)){ 
+	gdouble h=(event->x - mouseX)*(event->x - mouseX)+(event->y - mouseY)*(event->y - mouseY);
+	/*printf("h is %lf (%lf,%lf)--(%lf,%lf)\n",h,event->x ,event->y,mouseX,mouseY);*/
+	if (h < 25) double_click=TRUE;
+    }
+		
+	
+    if(double_click)
+    {	/* this works because item is selected at first click of double click */
+	skip_second_release=TRUE; /* skip the second button release event */
+	if (reselecttimer)g_source_remove(reselecttimer);
+		    /*printf("DBG:canceled reselecttimer = %d\n",reselecttimer );*/
+	reselecttimer=0;
+	
+
 	gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
 	if(gtk_tree_selection_get_selected(selection, &treemodel, &iter))
 	{
 	    gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
 	    newpath = g_strdup(en->path);
 	    type = en->type;
-	    if (IS_APP_TYPE(en->type) && !IS_ROOT_TYPE(en->type)){
-		double_click_run(tree_details,en);
-    		skip = TRUE;
-		goto done;
-	    }
-	    if (IS_FSTAB_TYPE(en->type) && IS_XF_FSTAB(en->type)){
-		    fstab_mount(treeview,en->path,IS_MOUNTED(en->type));	
+	    if ((IS_FSTAB_TYPE(en->type) && IS_XF_FSTAB(en->type))
+		    || IS_XF_NETSHARE(en->subtype)){
+		    fstab_mount(treeview,en->path,IS_MOUNTED(en->type),en);	
 		    goto done;
 	    }
 	    if(IS_DUMMY_TYPE(en->type) && en->path && strcmp(en->path, "..") == 0)
 	    {
-		/*tree_details->gogo = pushgo(treeview, newpath, tree_details->gogo);*/
 		tb_go_up((GtkButton *) tree_details->window, NULL);
 		goto done;
 	    }
@@ -690,31 +700,31 @@ gboolean treeclick(GtkWidget * widget, GdkEventButton * event, gpointer data)
 	    {
 		if (IS_DIR(en->type))
 		{
-		    tree_details->gogo = pushgo(treeview, newpath, tree_details->gogo);
+		    /*pushgo(treestuff, newpath);*/
 		    if (!IS_LOCAL_TYPE(en->type)){
     			GtkTreePath *treepath;
 			treepath = gtk_tree_model_get_path(treemodel, &iter);
 		    	gtk_tree_selection_unselect_path (selection,treepath);
 		    	gtk_tree_path_free(treepath);
 		    }
-		    go_to(treeview, newpath);
-		    /*go_to(tree_details->treeview, newpath);*/
+		    /* this is always an absolute path */
+		    go_to(treestuff, newpath);
 		}
 
 		else if(IS_EXE(en->type) )
 		{	/* run it (if not registered)*/
-			reg_t *prg = reg_prog_by_file (en->path);
-			if (prg) double_click_open_with(tree_details,en);
+			const gchar *prg = MIME_command (en->path);
+			if (prg) double_click_open_with(en);
 			else {
     				/*  assume in_term to be safe */
     				SET_IN_TERM(en->subtype);
-				double_click_run(tree_details,en);
+				double_click_run(en);
 			}
-    			skip = TRUE;
+			add2history(treeview, en->path);
 		} else 
 		{      /* open with */
-			double_click_open_with(tree_details,en);
-    			skip = TRUE;
+			double_click_open_with(en);
+			add2history(treeview, en->path);
 		}
 	    }
        	    else if(gtk_tree_model_iter_children(treemodel, &child, &iter)){
@@ -735,9 +745,18 @@ done:
     {
 	default:
 	    break;
+	case 2:		
+	    return TRUE;	
+	    /* this disables dnd (wrong), but also selection (right)*/
+#if 0
+	       return FALSE; 
+	    /* this enables dnd(right), but also selection(wrong)*/
+#endif
 	case 3:
 	    clear_dnd_selection_list(); /* only currently selected items */
-	    if (event->state!=GDK_CONTROL_MASK){
+	    /* if control not pressed, unselect other rows...*/
+	    /*if (event->state != GDK_CONTROL_MASK){*/
+	    if (!(event->state & GDK_CONTROL_MASK)){
     		gtk_tree_model_foreach(treemodel,unsel,selection);
 	    }
 	    if(gtk_tree_view_get_path_at_pos(treeview, event->x, 
@@ -757,38 +776,68 @@ done:
 	    }
 	    do_popup(treeview, event);
 	    return TRUE;
-	case 2:		
-	    return TRUE;	
-	    /* this disables dnd (wrong), but also selection (right)*/
-#if 0
-	       return FALSE; 
-	    /* this enables dnd(right), but also selection(wrong)*/
-#endif
-	case 1:
-	  if (event->state!=GDK_CONTROL_MASK && event->state!=GDK_SHIFT_MASK) {
+	case 1: 
+	/* case BUTTON_1: */
 	    clear_dnd_selection_list();
-    	    gtk_tree_selection_selected_foreach(selection, get_dnd_selection, (gpointer) treeview);
-	    if(gtk_tree_view_get_path_at_pos(treeview, event->x, 
-				    event->y, &treepath, 
-				    NULL, NULL, NULL))	
-	    if (!is_in_dnd_selection(treepath)){
+	    if(gtk_tree_view_get_path_at_pos(treeview, event->x, event->y, 
+				    &treepath, NULL, NULL, NULL))
+	    {
+		gtk_tree_model_get_iter(treemodel, &iter, treepath);
+		gtk_tree_model_get(treemodel, &iter, ENTRY_COLUMN, &en, -1);
+		update_status_line(en);	
+	        gtk_tree_path_free(treepath);
+	    }	
+	    if (tree_details->preferences & ENABLE_GTK_CLICK){
+		 g_timeout_add_full(0, 50, (GtkFunction) delayed_selection, (gpointer) treeview, NULL);
+	    } else { /* !ENABLE_GTK_CLICK */
+		gtk_tree_selection_selected_foreach(selection, get_dnd_selection, (gpointer) treeview);
+	        if (event->state & GDK_CONTROL_MASK || event->state & GDK_SHIFT_MASK){
+		    if (event->state & GDK_SHIFT_MASK){
+		       g_timeout_add_full(0, 50, (GtkFunction) delayed_selection, (gpointer) treeview, NULL);
+		    }
+		} else {
+		    reselecttimer = g_timeout_add_full(0, 150, (GtkFunction) do_reselect, (gpointer) treeview, NULL);
+		    /*printf("DBG:reselecttimer = %d\n",reselecttimer );*/
+		}
+	    }
+	    if(gtk_tree_view_get_path_at_pos(treeview, event->x,  event->y, &treepath, NULL, NULL, NULL))
+	    {	
+	      easy_mode = gtk_tree_selection_path_is_selected (selection,treepath);
+	      if (!is_in_dnd_selection(treepath)){
 		if (gtk_tree_model_get_iter (treemodel,&iter,treepath)) 
 		{
 		  get_dnd_selection(treemodel, treepath, &iter, (gpointer) treeview);
 		}
+		if (clickpath) gtk_tree_path_free(clickpath);
+		clickpath=gtk_tree_path_copy(treepath);
 		gtk_tree_path_free(treepath);
+	      } else {
+		if (clickpath) gtk_tree_path_free(clickpath);
+		clickpath=NULL;
+	      }
 	    }
-	  }
+#if DEBUG_IN_DND
+	/* DEBUG must be enabled in dnd.c for this to work: */
+#ifdef DEBUG
+	    {
+	      extern void the_sl(void);
+	      the_sl();
+	    }
+#endif
+#endif
 	    return FALSE;
     }
     return FALSE;
 }
 
+G_MODULE_EXPORT
 void on_double_click_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
     GdkEventButton event;
     event.button = 1;
     event.type = GDK_2BUTTON_PRESS;
+    event.x = mouseX;
+    event.y = mouseY;
     treeclick((GtkWidget *) menuitem, &event, user_data);
 }
 
@@ -803,51 +852,27 @@ static gboolean unsel(GtkTreeModel * treemodel,
     gtk_tree_selection_unselect_path (selection,treepath);
     return FALSE;
 }
-#if 0
-static gboolean selall(GtkTreeModel * treemodel, 
-		GtkTreePath * treepath, 
-		GtkTreeIter * iter, 
-		gpointer data)
-{
-    tree_entry_t *en;
-    gboolean *result=data;
-    gtk_tree_model_get(treemodel, iter, ENTRY_COLUMN, &en, -1);
-    if (title_en == en){
-	    GtkTreePath *tpath;
-	    /*printf("DBG: found!!!!!!\n");*/
-	    *result=TRUE;
-    	    if (title_reference) gtk_tree_row_reference_free(title_reference);
-       	    title_reference = gtk_tree_row_reference_new(treemodel, treepath);
-	    return TRUE;
-    } 
-    /*else {printf (".");fflush(NULL);}*/
-    *result=FALSE;    
-    return FALSE;
-}
-#endif
-
 
+G_MODULE_EXPORT
 void on_unselect_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
-    GtkTreeView *treeview = get_selected_treeview((GtkWidget *) menuitem);
-    GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
-    GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
+    gint tree_id = get_active_tree_id();
+    GtkTreeModel *treemodel = tree_details->treestuff[tree_id].treemodel;
+    GtkTreeSelection *selection = tree_details->treestuff[tree_id].selection;
     gtk_tree_model_foreach(treemodel,unsel,selection);
 
 }
 
+G_MODULE_EXPORT
 void on_select_all_activate(GtkMenuItem * menuitem, gpointer user_data)
 {
-    GtkTreeView *treeview = get_selected_treeview((GtkWidget *) menuitem);
-    GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
-    GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
+    gint tree_id = get_active_tree_id();
+    GtkTreeView *treeview = tree_details->treestuff[tree_id].treeview;
+    GtkTreeModel *treemodel = tree_details->treestuff[tree_id].treemodel;
+    GtkTreeSelection *selection = tree_details->treestuff[tree_id].selection;
     GtkTreeIter child,parent;
     tree_entry_t *en;
-    /*gboolean result;
-
-    	gtk_tree_model_foreach(treemodel,selall,&result);
-    	if (!result) return;*/
-    
+     
     	if (!title_reference) return;
 	if(!gtk_tree_row_reference_valid(title_reference))
 	   return ;
@@ -863,7 +888,6 @@ void on_select_all_activate(GtkMenuItem * menuitem, gpointer user_data)
 	    get_entry_from_reference(treeview, title_reference, &parent, &en);
 	}
     
-    
     	if (!gtk_tree_model_iter_children(treemodel,&child,&parent)) return;
     	do {
       	  gtk_tree_model_get(treemodel, &child, ENTRY_COLUMN, &en, -1);
@@ -874,4 +898,96 @@ void on_select_all_activate(GtkMenuItem * menuitem, gpointer user_data)
 }
 
 
+/***   properties_callbacks  ***/
+
+static void on_prop(GtkWidget *w){
+   GtkTreeView *treeview = tree_details->treestuff[get_active_tree_id()].treeview;
+   xfprop_functions *xfprop_fun = load_prop_module();
+   (*(xfprop_fun->do_prop))(treeview,tree_details->window);
+   unload_prop_module();
+   local_monitor(TRUE);
+}
+G_MODULE_EXPORT
+void on_properties_activate(GtkMenuItem * menuitem, gpointer user_data) {
+   on_prop((GtkWidget *)menuitem);
+}
+
+G_MODULE_EXPORT
+void tb_properties(GtkWidget * w) {
+   on_prop(w);
+}
+
+/*********   find callbacks   *********/
+
+
+G_MODULE_EXPORT
+void tb_find(GtkButton * button, gpointer user_data)
+{
+    xffind_functions *xffind_fun = load_find_module();
+    (*(xffind_fun->do_find))((GtkWidget *) button);
+    /* XXX: function returns before find is done, so
+     * we should not unload here, but later. 
+     * Can a gmodule unload itself when done? */
+    /*unload_find_module();*/
+}
+
+G_MODULE_EXPORT
+void sb_find(GtkWidget * widget)
+{
+    tb_find((GtkButton *)widget,NULL);
+}
+
+G_MODULE_EXPORT
+void on_find_activate(GtkMenuItem * menuitem, gpointer user_data)
+{
+    tb_find((GtkButton *)menuitem,NULL);
+}
+
+/* FIXME: this is duplicated above*/
+G_MODULE_EXPORT
+void on_view_find1_activate(GtkMenuItem * menuitem, gpointer user_data)
+{
+    tb_find((GtkButton *)menuitem,NULL);
+}
+
+
+
+G_MODULE_EXPORT
+void on_remove_from_results_activate(GtkMenuItem * menuitem, gpointer user_data)
+{
+
+    GtkTreeIter iter, parent;
+    tree_entry_t *en;
+    gint tree_id = get_active_tree_id();
+    GtkTreeView *treeview = tree_details->treestuff[tree_id].treeview;
+    GtkTreeModel *treemodel = tree_details->treestuff[tree_id].treemodel;
+    GtkTreeSelection *selection = tree_details->treestuff[tree_id].selection;
+
+    /*get selection */
+    gtk_tree_selection_set_mode(selection, GTK_SELECTION_BROWSE);
+    if(gtk_tree_selection_get_selected(selection, &treemodel, &iter))
+    {
+	remove_row(treemodel, &iter,NULL,NULL);
+    }
+    gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
+    get_find_root(treeview, &parent, &en);
+    insert_dummy_row(treemodel, &parent,NULL,en,"xfce/info",_("Nothing searched"));
+}
+
+
+G_MODULE_EXPORT
+void on_clear_all_results_activate(GtkMenuItem * menuitem, gpointer user_data)
+{
+    GtkTreeIter parent;
+    tree_entry_t *en;
+    gint tree_id = get_active_tree_id();
+    GtkTreeView *treeview = tree_details->treestuff[tree_id].treeview;
+    GtkTreeModel *treemodel = tree_details->treestuff[tree_id].treemodel;
+
+    get_find_root(treeview, &parent, &en);
+    prune_row(treemodel,&parent,NULL,en);
+    insert_dummy_row(treemodel, &parent,NULL,en,"xfce/info",_("Nothing searched"));
+
+}
+
 
