@@ -1,4 +1,4 @@
-/*      $Id: placement.c,v 1.17 2004/08/28 23:10:17 olivier Exp $
+/*      $Id: placement.c,v 1.19 2004/10/20 17:47:52 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -224,7 +224,7 @@ clientConstrainPos (Client * c, gboolean show_full)
     cy = frame_y + (frame_height / 2);
 
     screen_info = c->screen_info;
-    monitor_nbr = gdk_screen_get_monitor_at_point (screen_info->gscr, cx, cy);
+    monitor_nbr = find_monitor_at_point (screen_info->gscr, cx, cy);
     gdk_screen_get_monitor_geometry (screen_info->gscr, monitor_nbr, &rect);
 
     disp_x = rect.x;
@@ -268,20 +268,26 @@ clientConstrainPos (Client * c, gboolean show_full)
                 && (c2 != c))
             {
                 /* Right */
-                if (overlapY (frame_y, frame_y + frame_height, c2->struts[RIGHT_START_Y], c2->struts[RIGHT_END_Y]))
+                if (overlapY (frame_y, frame_y + frame_height, 
+                              c2->struts[RIGHT_START_Y], c2->struts[RIGHT_END_Y]))
                 {
                     if (overlapX (frame_x, frame_x + frame_width, 
-                                  gdk_screen_get_width (screen_info->gscr) - c2->struts[RIGHT], gdk_screen_get_width (screen_info->gscr)))
+                                  gdk_screen_get_width (screen_info->gscr) - c2->struts[RIGHT], 
+                                  gdk_screen_get_width (screen_info->gscr)))
                     {
-                        c->x = gdk_screen_get_width (screen_info->gscr) - c2->struts[RIGHT] - frame_width;
+                        /* Coord shouldn't shrink, thus the MAX, otherwise you end up off screen */
+                        c->x = MAX (gdk_screen_get_width (screen_info->gscr) - c2->struts[RIGHT] - frame_width, 
+                                    c->x);
                         frame_x = frameX (c);
                     }
                 }
 
                 /* Left */
-                if (overlapY (frame_y, frame_y + frame_height, c2->struts[LEFT_START_Y], c2->struts[LEFT_END_Y]))
+                if (overlapY (frame_y, frame_y + frame_height, 
+                              c2->struts[LEFT_START_Y], c2->struts[LEFT_END_Y]))
                 {
-                    if (overlapX (frame_x, frame_x + frame_width, 0, c2->struts[LEFT]))
+                    if (overlapX (frame_x, frame_x + frame_width, 
+                                  0, c2->struts[LEFT]))
                     {
                         c->x = c2->struts[LEFT] + frame_left;
                         frame_x = frameX (c);
@@ -289,20 +295,28 @@ clientConstrainPos (Client * c, gboolean show_full)
                 }
 
                 /* Bottom */
-                if (overlapX (frame_x, frame_x + frame_width, c2->struts[BOTTOM_START_X], c2->struts[BOTTOM_END_X]))
+                if (overlapX (frame_x, frame_x + frame_width, 
+                              c2->struts[BOTTOM_START_X], c2->struts[BOTTOM_END_X]))
                 {
                     if (overlapY (frame_y, frame_y + frame_height, 
-                        gdk_screen_get_height (screen_info->gscr) - c2->struts[BOTTOM], gdk_screen_get_height (screen_info->gscr)))
+                                  gdk_screen_get_height (screen_info->gscr) - c2->struts[BOTTOM], 
+                                  gdk_screen_get_height (screen_info->gscr)))
                     {
-                        c->y = gdk_screen_get_height (screen_info->gscr) - c2->struts[BOTTOM] - frame_height;
+                        /* Coord shouldn't shrink, thus the MAX, otherwise you end up off screen */
+                        c->y = MAX (gdk_screen_get_height (screen_info->gscr) - c2->struts[BOTTOM] - frame_height,
+                                    c->y);
                         frame_y = frameY (c);
                     }
                 }
 
                 /* Top */
-                if (overlapX (frame_x, frame_x + frame_width, c2->struts[TOP_START_X], c2->struts[TOP_END_X]))
+                if (overlapX (frame_x, 
+                              frame_x + frame_width, 
+                              c2->struts[TOP_START_X], 
+                              c2->struts[TOP_END_X]))
                 {
-                    if (overlapY (frame_y, frame_y + frame_height, 0, c2->struts[TOP]))
+                    if (overlapY (frame_y, frame_y + frame_height, 
+                                  0, c2->struts[TOP]))
                     {
                         c->y = c2->struts[TOP] + frame_top;
                         frame_y = frameY (c);
@@ -346,7 +360,8 @@ clientConstrainPos (Client * c, gboolean show_full)
                 && (c2 != c))
             {
                 /* Right */
-                if (overlapY (frame_y, frame_y + frame_height, c2->struts[RIGHT_START_Y], c2->struts[RIGHT_END_Y]))
+                if (overlapY (frame_y, frame_y + frame_height, 
+                              c2->struts[RIGHT_START_Y], c2->struts[RIGHT_END_Y]))
                 {
                     if (frame_x > gdk_screen_get_width (screen_info->gscr) - c2->struts[RIGHT] - CLIENT_MIN_VISIBLE)
                     {
@@ -356,7 +371,8 @@ clientConstrainPos (Client * c, gboolean show_full)
                 }
 
                 /* Left */
-                if (overlapY (frame_y, frame_y + frame_height, c2->struts[LEFT_START_Y], c2->struts[LEFT_END_Y]))
+                if (overlapY (frame_y, frame_y + frame_height, 
+                              c2->struts[LEFT_START_Y], c2->struts[LEFT_END_Y]))
                 {
                     if (frame_x + frame_width < c2->struts[LEFT] + CLIENT_MIN_VISIBLE)
                     {
@@ -366,7 +382,8 @@ clientConstrainPos (Client * c, gboolean show_full)
                 }
 
                 /* Bottom */
-                if (overlapX (frame_x, frame_x + frame_width, c2->struts[BOTTOM_START_X], c2->struts[BOTTOM_END_X]))
+                if (overlapX (frame_x, frame_x + frame_width, 
+                              c2->struts[BOTTOM_START_X], c2->struts[BOTTOM_END_X]))
                 {
                     if (frame_y > gdk_screen_get_height (screen_info->gscr) - c2->struts[BOTTOM] - CLIENT_MIN_VISIBLE)
                     {
@@ -376,7 +393,8 @@ clientConstrainPos (Client * c, gboolean show_full)
                 }
 
                 /* Top */
-                if (overlapX (frame_x, frame_x + frame_width, c2->struts[TOP_START_X], c2->struts[TOP_END_X]))
+                if (overlapX (frame_x, frame_x + frame_width, 
+                              c2->struts[TOP_START_X], c2->struts[TOP_END_X]))
                 {
                     if (overlapY (frame_y, frame_y + frame_top, 0, c2->struts[TOP]))
                     {
@@ -429,7 +447,7 @@ clientKeepVisible (Client * c)
          */
         getMouseXY (c->screen_info, c->screen_info->xroot, &cx, &cy);
         
-        monitor_nbr = gdk_screen_get_monitor_at_point (c->screen_info->gscr, cx, cy);
+        monitor_nbr = find_monitor_at_point (c->screen_info->gscr, cx, cy);
         gdk_screen_get_monitor_geometry (c->screen_info->gscr, monitor_nbr, &rect);
 
         c->x = rect.x + (rect.width - c->width) / 2;
@@ -483,7 +501,7 @@ clientInitPosition (Client * c)
 
     getMouseXY (screen_info, screen_info->xroot, &msx, &msy);
 
-    monitor_nbr = gdk_screen_get_monitor_at_point (screen_info->gscr, msx, msy);
+    monitor_nbr = find_monitor_at_point (screen_info->gscr, msx, msy);
     gdk_screen_get_monitor_geometry (screen_info->gscr, monitor_nbr, &rect);
     
     frame_x = frameX (c);
