@@ -1,4 +1,4 @@
-/*      $Id: stacking.c,v 1.22 2004/09/23 17:15:38 olivier Exp $
+/*      $Id: stacking.c,v 1.24 2004/10/27 16:56:54 olivier Exp $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -54,7 +54,7 @@ clientApplyStackList (ScreenInfo *screen_info)
     {
         GList *index = NULL;
         Client *c = NULL;
-        
+
         for (index = g_list_last(screen_info->windows_stack); index; index = g_list_previous (index))
         {
             c = (Client *) index->data;
@@ -64,53 +64,21 @@ clientApplyStackList (ScreenInfo *screen_info)
     }
 
     XRestackWindows (myScreenGetXDisplay (screen_info), xwinstack, (int) nwindows + 4);
-    
+
     g_free (xwinstack);
 }
 
-gboolean
-clientTransientOrModalHasAncestor (Client * c, int ws)
-{
-    Client *c2 = NULL;
-    GList *index = NULL;
-    ScreenInfo *screen_info = NULL;
-    
-    g_return_val_if_fail (c != NULL, FALSE);
-
-    TRACE ("entering clientTransientOrModalHasAncestor");
-
-    if (!clientIsTransientOrModal (c))
-    {
-        return FALSE;
-    }
-
-    screen_info = c->screen_info;
-    for (index = screen_info->windows_stack; index; index = g_list_next (index))
-    {
-        c2 = (Client *) index->data;
-        if ((c2 != c) && !clientIsTransientOrModal (c2)
-            && clientIsTransientOrModalFor (c, c2)
-            && !FLAG_TEST (c2->flags, CLIENT_FLAG_ICONIFIED)
-            && (c2->win_workspace == ws))
-        {
-            return TRUE;
-        }
-    }
-    return FALSE;
-
-}
-
 Client *
 clientGetLowestTransient (Client * c)
 {
     Client *lowest_transient = NULL, *c2;
     GList *index = NULL;
     ScreenInfo *screen_info = NULL;
-    
+
     g_return_val_if_fail (c != NULL, NULL);
 
     TRACE ("entering clientGetLowestTransient");
-    
+
     screen_info = c->screen_info;
     for (index = screen_info->windows_stack; index; index = g_list_next (index))
     {
@@ -375,7 +343,7 @@ clientRaise (Client * c)
             g_list_free (windows_stack_copy);
         }
         /* Now, screen_info->windows_stack contains the correct window stack
-           We still need to tell the X Server to reflect the changes 
+           We still need to tell the X Server to reflect the changes
          */
         clientApplyStackList (screen_info);
         clientSetNetClientList (c->screen_info, net_client_list_stacking, screen_info->windows_stack);
@@ -401,7 +369,7 @@ clientLower (Client * c)
     if (FLAG_TEST (c->xfwm_flags, XFWM_FLAG_MANAGED))
     {
         Client *client_sibling = NULL;
-        
+
         if (clientIsTransientOrModalForGroup (c))
         {
             client_sibling = clientGetTopMostForGroup (c);
@@ -424,7 +392,7 @@ clientLower (Client * c)
                 gint position = g_list_position (screen_info->windows_stack, sibling) + 1;
 
                 screen_info->windows_stack = g_list_insert (screen_info->windows_stack, c, position);
-                TRACE ("lowest client is \"%s\" (0x%lx) at position %i", 
+                TRACE ("lowest client is \"%s\" (0x%lx) at position %i",
                         client_sibling->name, client_sibling->window, position);
             }
             else
@@ -433,7 +401,7 @@ clientLower (Client * c)
             }
         }
         /* Now, screen_info->windows_stack contains the correct window stack
-           We still need to tell the X Server to reflect the changes 
+           We still need to tell the X Server to reflect the changes
          */
         clientApplyStackList (screen_info);
         clientSetNetClientList (screen_info, net_client_list_stacking, screen_info->windows_stack);
@@ -449,10 +417,10 @@ gboolean
 clientAdjustFullscreenLayer (Client *c, gboolean set)
 {
     g_return_val_if_fail (c, FALSE);
-       
+
     if (set)
     {
-        if (FLAG_TEST(c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN) 
+        if (FLAG_TEST(c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN)
             || FLAG_TEST(c->flags, CLIENT_FLAG_FULLSCREEN))
         {
             clientSetLayer (c, WIN_LAYER_ABOVE_DOCK);
@@ -461,7 +429,7 @@ clientAdjustFullscreenLayer (Client *c, gboolean set)
     }
     else if (c->win_layer == WIN_LAYER_ABOVE_DOCK)
     {
-        if (FLAG_TEST(c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN) 
+        if (FLAG_TEST(c->xfwm_flags, XFWM_FLAG_LEGACY_FULLSCREEN)
             || FLAG_TEST(c->flags, CLIENT_FLAG_FULLSCREEN))
         {
             if (FLAG_TEST(c->flags, CLIENT_FLAG_FULLSCREEN))
@@ -490,7 +458,7 @@ clientAddToList (Client * c)
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
     myDisplayAddClient (display_info, c);
-    
+
     screen_info->client_count++;
     if (screen_info->clients)
     {
@@ -513,7 +481,7 @@ clientAddToList (Client * c)
     clientSetNetClientList (screen_info, net_client_list, screen_info->windows);
     clientSetNetClientList (screen_info, win_client_list, screen_info->windows);
     clientSetNetClientList (screen_info, net_client_list_stacking, screen_info->windows_stack);
-  
+
     FLAG_SET (c->xfwm_flags, XFWM_FLAG_MANAGED);
 }
 
@@ -522,7 +490,7 @@ clientRemoveFromList (Client * c)
 {
     ScreenInfo *screen_info = NULL;
     DisplayInfo *display_info = NULL;
- 
+
     g_return_if_fail (c != NULL);
     TRACE ("entering clientRemoveFromList");
 
@@ -531,7 +499,7 @@ clientRemoveFromList (Client * c)
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
     myDisplayRemoveClient (display_info, c);
-    
+
     g_assert (screen_info->client_count > 0);
     screen_info->client_count--;
     if (screen_info->client_count == 0)
@@ -579,7 +547,7 @@ void
 clientSetLastRaise (Client *c)
 {
     ScreenInfo *screen_info = NULL;
- 
+
     g_return_if_fail (c != NULL);
 
     screen_info = c->screen_info;
