@@ -638,8 +638,9 @@ mk_smart_button(	smart_button_init_t *button_info,
 
 
       g_object_unref(G_OBJECT(pixbuf));
-      if (smart->type == SMART_COLUMN_BUTTON) insert_pixbuf_tag("downarrow.png",&p, GTK_ICON_SIZE_SMALL_TOOLBAR,3,"SE");
-      else insert_pixbuf_tag("downarrow.png",&p, GTK_ICON_SIZE_SMALL_TOOLBAR,4,"SE"); 
+     /* if (smart->type == SMART_COLUMN_BUTTON) insert_pixbuf_tag("downarrow.png",&p, GTK_ICON_SIZE_SMALL_TOOLBAR,3,"SE");
+      else */
+      insert_pixbuf_tag("downarrow.png",&p, GTK_ICON_SIZE_SMALL_TOOLBAR,3,"SE"); 
       smart->smart_button->altimage = gtk_image_new_from_pixbuf (p);
       g_object_unref(p);
       gtk_widget_show (smart->smart_button->altimage);
@@ -675,13 +676,16 @@ mk_smart_button(	smart_button_init_t *button_info,
       	pixbuf = icon_tell(GTK_ICON_SIZE_SMALL_TOOLBAR,button_info[lastused].image);
   }
   {
-    GdkPixbuf *p;
-    if (smart->type == SMART_COLUMN_BUTTON) p=gdk_pixbuf_scale_simple ((const GdkPixbuf *)pixbuf,16,16,GDK_INTERP_BILINEAR);
-    else p=gdk_pixbuf_copy(pixbuf);
+    GdkPixbuf *p=NULL;
+    if (pixbuf) {
+	if (smart->type == SMART_COLUMN_BUTTON) p=gdk_pixbuf_scale_simple ((const GdkPixbuf *)pixbuf,16,16,GDK_INTERP_BILINEAR);
+	else p=gdk_pixbuf_copy(pixbuf);
 
-    if (pixbuf) g_object_unref(G_OBJECT(pixbuf));
-    if (smart->type == SMART_COLUMN_BUTTON) insert_pixbuf_tag("xfce/downarrow",&p, GTK_ICON_SIZE_SMALL_TOOLBAR,3,"SE"); 
-    else insert_pixbuf_tag("xfce/downarrow",&p, GTK_ICON_SIZE_SMALL_TOOLBAR,4,"SE");
+	g_object_unref(G_OBJECT(pixbuf));
+    }
+    /*if (smart->type == SMART_COLUMN_BUTTON) insert_pixbuf_tag("downarrow.png",&p, GTK_ICON_SIZE_SMALL_TOOLBAR,3,"SE"); 
+    else */
+    insert_pixbuf_tag("downarrow.png",&p, GTK_ICON_SIZE_SMALL_TOOLBAR,3,"SE");
     smart->image = gtk_image_new_from_pixbuf (p);
     if (p) g_object_unref(G_OBJECT(p));
   }
@@ -905,6 +909,8 @@ void toolbar_buttons(void){
     /* separators added in normal button info */
     /*gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));*/
     normal_buttons(toolbar,normal_button_info1);
+    smart_buttons(toolbar,paste_button_info,"paste_button");
+    normal_buttons(toolbar,normal_button_info1b);
     smart_buttons(toolbar,new_button_info,"new_button");
     smart_buttons(toolbar,single_selection_button_info,"single_selection_button");
     /* separator is added in normal button info */
@@ -1203,38 +1209,34 @@ put_in_menu(		const gchar *menutext,
 			GtkWidget *parent_menu,
 			void *callback,
 			const gchar *name,
-			gchar key,
-			GtkAccelGroup **accel_group)
+			guint key)
 {
     gchar *menuname;
     GtkWidget *image=NULL;
     GtkWidget *menuitem = gtk_image_menu_item_new_with_mnemonic (_(menutext));
     gtk_widget_show (menuitem);
 
- /*   if (!accel_group) {
-	GSList*    list=gtk_accel_groups_from_object((GObject *)(GTK_WINDOW (tree_details->window)));
-	if (!list || !list->data) assert_not_reached();
-	accel_group = (GtkAccelGroup *)list->data;
-    }*/
-
-
-    
     gtk_container_add (GTK_CONTAINER (parent_menu), menuitem);
     /*XXX: probably should register all xffm stock icons as
      *       stock icons to use only gtk_image_new_from_stock,
      *       but apparently not much is gained, except the
      *       overhead from doing do
      *       */
+
     if (icon_id) {
 	if (strncmp(icon_id,"gtk-",strlen("gtk-"))==0){
 	  GdkPixbuf *p =load_stock_icon(icon_id, GTK_ICON_SIZE_MENU);
-	  image = gtk_image_new_from_pixbuf (p);
-	  g_object_unref(p);
+	  if (p) {
+	    image = gtk_image_new_from_pixbuf (p);
+	    g_object_unref(p);
+	  }
 	  /*image = gtk_image_new_from_stock (icon_id, GTK_ICON_SIZE_MENU);*/
 	} else {
     	  GdkPixbuf *p = icon_tell(GTK_ICON_SIZE_MENU,icon_id);
-	  image = gtk_image_new_from_pixbuf (p);
-	  g_object_unref(p);
+	  if (p) {
+	    image = gtk_image_new_from_pixbuf (p);
+	    g_object_unref(p);
+	  }
 	}
 	if (image) {
 	  gtk_widget_show (image);
@@ -1244,11 +1246,17 @@ put_in_menu(		const gchar *menutext,
     if (callback) {
 	g_signal_connect ((gpointer) menuitem, "activate",G_CALLBACK (callback), NULL);
 	if (key) {
+	    GSList *l;
+	    l = gtk_accel_groups_from_object((GObject *) tree_details->window);
+#ifdef DEBUG
 	    printf("DBG: adding CTL+%c\n",key);
-	    if (!*accel_group) *accel_group = gtk_accel_group_new ();
-	    gtk_widget_add_accelerator (menuitem, "activate", *accel_group,
+#endif
+	    if(l && l->data){
+		gtk_widget_add_accelerator (menuitem, "activate", 
+			    (GtkAccelGroup *) (l->data),
                               key, GDK_CONTROL_MASK,
                               GTK_ACCEL_VISIBLE);
+	    }
 	}
     }
     if (name) {
@@ -1262,15 +1270,7 @@ void
 mk_smart_menu(		GtkWidget *parent_menu,
 			smart_button_init_t *button_info)
 {
-  static GtkAccelGroup *accel_group=NULL;
   int i;
-  
-  if (!accel_group) {
-      accel_group = gtk_accel_group_new ();
-      gtk_window_add_accel_group (GTK_WINDOW (tree_details->window), accel_group);
-    printf("FIXME: why isn't this accel group responding?\n");
-  } 
-
   for (i=0; button_info[i].tooltip; i++){
     if (strcmp(button_info[i].tooltip,"-")==0){
 	GtkWidget *separator = gtk_menu_item_new ();
@@ -1280,14 +1280,14 @@ mk_smart_menu(		GtkWidget *parent_menu,
         if (button_info[i].name)
 	    GLADE_HOOKUP_OBJECT (tree_details->window, separator, button_info[i].name);
 	continue;
-    } else if (button_info[i].menu_callback) {
+    } 
+    else if (button_info[i].menu_callback) {
 	put_in_menu(	(const gchar *)button_info[i].tooltip,
 			(const gchar *)button_info[i].image,
 			parent_menu,
 			button_info[i].menu_callback,
 			(const gchar *)button_info[i].name,
-			button_info[i].key,
-			&accel_group);
+			button_info[i].key);
 			
 	
 
