@@ -38,12 +38,14 @@
 #include <glib.h>
 #include <gmodule.h>
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/xfce-icontheme.h>
+#include <libxfcegui4/libxfcegui4.h>
 
 /* include non installed headers */
 #include <xfce4-modules/headers/constants.h>
 #include <xfce4-modules/headers/mime_icons.h>
 
+#include "icon-theme.h"
+
 #ifdef DEBUG
 #undef DEBUG
 #endif
@@ -53,20 +55,10 @@
 
 /*extern GtkWidget *main_window;*/
 
-#ifndef HAVE_LIBXML
-#warning "mime_icons module will be disabled without libxml!"
-G_MODULE_EXPORT
-const gchar *g_module_check_init(GModule *module){
-    g_error("Cannot work with mime-icons module: libxfcegui4 constructed without libxml.");
-    return NULL;
-}
-#else
-
 static GHashTable *theme_hash = NULL;
 static GHashTable *icon_hash = NULL;
 static GtkIconFactory *icon_factory = NULL;
 /*static GList *pixmaps_directories = NULL;*/
-static XfceIconTheme *icon_theme=NULL;
 static gchar *icon_theme_name=NULL;
 
 typedef struct theme_info_t {
@@ -87,7 +79,7 @@ GtkIconSet *mime_icon_get_iconset(const gchar *id, GtkWidget *main_window);
 G_MODULE_EXPORT
 void mime_icon_add_iconset(const gchar *tag, GtkIconSet *icon_set);
 G_MODULE_EXPORT
-GHashTable * mime_icon_load_theme(void);
+int mime_icon_load_theme(void);
 
 G_MODULE_EXPORT
 char **mime_icon_find_themes(gboolean only_valid,gboolean full_paths);
@@ -126,8 +118,10 @@ G_MODULE_EXPORT
 gchar *
 mime_icon_get_local_xml_file(const gchar *theme_name){
     gchar *xdg_dir= xfce_resource_save_location (XFCE_RESOURCE_THEMES,"/",TRUE);
-    gchar *n=g_build_filename(xdg_dir,theme_name,NULL); 
+    gchar *base=g_path_get_basename(theme_name);
+    gchar *n=g_build_filename(xdg_dir,base,NULL); 
     gchar *mimefile=g_strconcat(n,".mime.xml",NULL);
+    g_free(base);
     g_free(n);
     g_free(xdg_dir);
     return mimefile;
@@ -339,8 +333,11 @@ static void clear_iconhash(gpointer key, gpointer value, gpointer user_data)
 
 gchar*
 mime_icon_find_pixmap_file                       (const gchar     *filename)
-{    
-  return (xfce_icon_theme_lookup (icon_theme, filename, 48));
+{   
+    const gchar *path=find_icon_path(filename, 48,"MimeType");
+    if (!path) return NULL;
+    else return (g_strdup(path));
+  /*return (g_strdup(find_icon_path(filename, 48,NULL)));*/
 }
 
 
@@ -449,16 +446,20 @@ mime_icon_get_global_xml_file(const gchar *theme_name){
 	g_strfreev(inherits);  
 	
    }
-   if (!mimefile || access(mimefile,F_OK)!=0) {
-        g_free(mimefile); mimefile=NULL;
-	g_warning("No mime file found for theme %s\n",theme_name);
-	return mime_icon_get_global_xml_file("hicolor");
+   if (!mimefile || access(mimefile,F_OK)!=0){
+	g_warning("No mime file found for theme %s",theme_name);
+        if (strcmp("hicolor",theme_name)) {
+	    g_free(mimefile); mimefile=NULL;
+	    return mime_icon_get_global_xml_file("hicolor");
+	} else {
+	    return NULL;
+	}
    }	
     
    return mimefile;
 }
 
-static gboolean create_icon_hash(void){
+static int create_icon_hash(void){
     gchar *mimefile = NULL;
 
     theme_info_t *theme_info;  
@@ -468,9 +469,6 @@ static gboolean create_icon_hash(void){
     /*g_free(icon_theme_name);*/
 
     
-    icon_theme = xfce_icon_theme_get_for_screen(NULL);
-    
-    /* this should be the same theme as above */
 #if GTK_CHECK_VERSION (2,4,0)
     g_object_get (gtk_settings_get_default(), "gtk-icon-theme-name", &icon_theme_name, NULL);
 #else /* gtk < 2.4 */
@@ -479,30 +477,40 @@ static gboolean create_icon_hash(void){
       g_object_get (G_OBJECT (gsettings), "gtk-icon-theme-name", &icon_theme_name, NULL);
     }
 #endif
-  
+    if (!icon_theme_name) 
+	icon_theme_name="hicolor";
+    /*printf("hey: iconthemename=%s\n",icon_theme_name);*/
+
+
+    if (open_theme(icon_theme_name)==-1) {
+	D(printf("icon_hash already exists...\n");)
+	return -1;
+    }
+    
+	
 
    /* test for user defined mimefile (via xfmime-edit)*/
    mimefile= mime_icon_get_local_xml_file(icon_theme_name);
-    D(printf("create_icon_hash...%s\n",mimefile);)
+    D(printf("create_icon_hash from local...%s\n",mimefile);)
 
    /* test for default mimefile (xfce distributed)  */
    if (!mimefile || !g_file_test(mimefile,G_FILE_TEST_EXISTS)) {
        g_free(mimefile);
 	mimefile=mime_icon_get_global_xml_file(icon_theme_name);
+      D(printf("create_icon_hash from global ...3\n");)
    }
-    D(printf("create_icon_hash...3\n");)
    
    if (!mimefile || !g_file_test(mimefile,G_FILE_TEST_EXISTS)) {
-       g_warning("no mime theme found for %s",icon_theme_name);
+       /*g_message("no mime theme found for %s",icon_theme_name);*/
        g_free(mimefile);
-       return FALSE;
+       if (strcmp("hicolor",icon_theme_name)==0) return 1;
+       else return 0;
    }
    
     if (theme_hash==NULL) {
 	    D(printf("DBG:creating new theme_hash...\n");)
 	    theme_hash = g_hash_table_new(g_str_hash, g_str_equal);
     }
-    D(printf("create_icon_hash...4\n");)
 
     theme_info = g_hash_table_lookup(theme_hash,icon_theme_name);
     if (!theme_info){
@@ -516,16 +524,15 @@ static gboolean create_icon_hash(void){
 	    icon_hash = theme_info->icon_hash;
 	    icon_factory = theme_info->icon_factory;
 	    g_free(mimefile);
-	    return TRUE;
+	    return 1;
     }
     
-    D(printf("create_icon_hash...5\n");)
     glib_parser(mimefile);
    
     g_free(mimefile);mimefile=NULL;
 	    
     D(printf("DBG:theme_hash created!\n");)
-    return TRUE;
+    return 1;
 }
 
 
@@ -535,21 +542,26 @@ static gboolean create_icon_hash(void){
 G_MODULE_EXPORT
 GdkPixbuf *mime_icon_create_pixbuf                          (const gchar     *filename)
 {
+  GdkPixbuf *pixbuf;
+  GError *error=NULL;
+  const gchar *fullpath;
   if (!filename || !filename[0]) return NULL;
-
-  /*if (strstr(filename,"directory"))printf("dbg: lookup(%s)=%s\n",filename,xfce_icon_theme_lookup (icon_theme, filename, 48));*/
-  return xfce_icon_theme_load(icon_theme, filename,48);
+  if ((fullpath = find_icon_path(filename,48,"MimeType"))==NULL) return NULL;
+  /*printf("DBG: creating pixbuf for... %s\n",fullpath);*/
+  pixbuf = xfce_pixbuf_new_from_file_at_size(fullpath, 48,48,&error);
+  if (error) g_error_free(error);
+  return pixbuf;
 }
 
 G_MODULE_EXPORT
 GtkWidget *mime_icon_create_pixmap (GtkWidget *widget, const gchar     *filename)
 {
-  gchar *pathname = NULL;
+  const gchar *pathname;
   GtkWidget *pixmap;
 
   if (!filename || !filename[0]) return gtk_image_new ();
-  /*if (strstr(filename,"directory"))printf("dbg: lookup(%s)=%s\n",filename,xfce_icon_theme_lookup (icon_theme, filename, 48));*/
-  pathname = xfce_icon_theme_lookup (icon_theme, filename, 48);
+  pathname = (gchar *)find_icon_path(filename, 48, "MimeType");
+  /*pathname = (gchar *)find_icon_path(filename, 48, NULL);*/
   
 
   if (!pathname)
@@ -560,7 +572,6 @@ GtkWidget *mime_icon_create_pixmap (GtkWidget *widget, const gchar     *filename
       return gtk_image_new ();
     }
   pixmap = gtk_image_new_from_file (pathname);
-  g_free (pathname);
   if (!pixmap) {
       g_warning("FIXME: this should not happen (gtk<2.4 and no libsvg");
       return gtk_image_new ();
@@ -580,12 +591,7 @@ GtkIconSet *mime_icon_get_iconset(const gchar *id, GtkWidget *main_window){
 #endif
     if (!id || !strlen(id)) return NULL;
 
-    if (!icon_hash) {
-	if (!create_icon_hash()) {
-	   g_error("!create_icon_hash()");
-	   return NULL;
-	}
-    }
+    if (!icon_hash) return NULL;
     icon_set = gtk_icon_factory_lookup(icon_factory, id);
     if (icon_set) return icon_set;
 
@@ -662,16 +668,17 @@ void g_module_unload(GModule *module){
 	gtk_icon_factory_remove_default (icon_factory);
 	if (xfmime_icon_fun) g_free(xfmime_icon_fun);
 	xfmime_icon_fun=NULL;
+	close_theme();
 }
 
 G_MODULE_EXPORT
-GHashTable * mime_icon_load_theme(void){
-    gboolean r;
+int mime_icon_load_theme(void){
+    int r;
     D(printf("mime_icon_load_theme...\n");)
     r=create_icon_hash();
-    if (r) return icon_hash;
-    g_warning("cannot create icon_hash");
-    return NULL;
+    if (r) return r;
+    g_message("cannot create icon_hash");
+    return r;
 }
 
 G_MODULE_EXPORT
@@ -679,4 +686,3 @@ xfmime_icon_functions *module_init(void){
     return xfmime_icon_fun;
 }
 
-#endif
