@@ -1,4 +1,4 @@
-/* $Id: xfsm-manager.c,v 1.20 2004/08/22 15:03:52 benny Exp $ */
+/* $Id: xfsm-manager.c,v 1.22 2004/10/05 12:54:55 benny Exp $ */
 /*-
  * Copyright (c) 2003-2004 Benedikt Meurer <benny@xfce.org>
  * All rights reserved.
@@ -448,9 +448,21 @@ xfsm_manager_init (XfceRc *rc)
 {
   gchar *display_name;
   gchar *resource_name;
+#ifdef HAVE_OS_CYGWIN
+  gchar *s;
+#endif
 
   xfce_rc_set_group (rc, "General");
   display_name  = xfce_gdk_display_get_fullname (gdk_display_get_default ());
+
+#ifdef HAVE_OS_CYGWIN
+  /* rename a colon (:) to a hash (#) under cygwin. windows doesn't like
+   * filenames with a colon... */
+  for (s = display_name; *s != '\0'; ++s)
+    if (*s == ':')
+      *s = '#';
+#endif
+
   resource_name = g_strconcat ("sessions/xfce4-session-", display_name, NULL);
   session_file  = xfce_resource_save_location (XFCE_RESOURCE_CACHE, resource_name, TRUE);
   g_free (resource_name);
