@@ -43,6 +43,7 @@
 #include <stdarg.h>
 #include "xfdiff.h"
 #include "xfdiff_misc.h"
+#include "modules.h"
 
 #ifdef XFCE4_O_X_LIBRARIES
 #warning "Using compatibility to xfce-4.0.x libraries (this should be removed for 4.2 release)"
@@ -833,7 +834,7 @@ cb_about (GtkWidget * widget, gpointer data)
 #ifndef XFCE4_O_X_LIBRARIES
     XfceAboutInfo *info;
     GtkWidget *dialog;
-    GdkPixbuf *icon =  create_pixbuf ("xfdiff.png");
+    GdkPixbuf *icon =   MIME_ICON_create_pixbuf ("xfdiff.png");
     if (!icon) printf("NULL icon\n");
     info = xfce_about_info_new(
 	    "XFDiff4",
@@ -1342,7 +1343,7 @@ create_diff_window (void)
   diff = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   {
    GdkPixbuf *icon_pixbuf;
-   icon_pixbuf = create_pixbuf ("xfdiff.png");
+   icon_pixbuf = MIME_ICON_create_pixbuf ("xfdiff.png");
    if (icon_pixbuf){
       gtk_window_set_icon (GTK_WINDOW (diff), icon_pixbuf);
       g_object_unref (G_OBJECT(icon_pixbuf));
